@@ -18,8 +18,10 @@
  */
 package org.apache.empire.jsf2.app;
 
+import java.io.IOException;
 import java.sql.Connection;
 import java.sql.SQLException;
+import java.text.MessageFormat;
 import java.util.HashMap;
 import java.util.Iterator;
 import java.util.Locale;
@@ -29,12 +31,13 @@ import java.util.ResourceBundle;
 import javax.faces.application.Application;
 import javax.faces.application.FacesMessage;
 import javax.faces.application.FacesMessage.Severity;
-import javax.faces.application.ViewExpiredException;
 import javax.faces.component.NamingContainer;
 import javax.faces.component.UIComponent;
 import javax.faces.component.UIViewRoot;
 import javax.faces.context.ExternalContext;
 import javax.faces.context.FacesContext;
+import javax.faces.event.ExceptionQueuedEvent;
+import javax.faces.event.ExceptionQueuedEventContext;
 import javax.servlet.ServletContext;
 import javax.servlet.http.HttpServletRequest;
 import javax.sql.DataSource;
@@ -44,15 +47,17 @@ import org.apache.empire.data.DataType;
 import org.apache.empire.db.DBDatabase;
 import org.apache.empire.db.context.DBRollbackManager;
 import org.apache.empire.db.context.DBRollbackManager.ReleaseAction;
-import org.apache.empire.db.exceptions.UserLevelException;
+import org.apache.empire.exceptions.EmpireException;
 import org.apache.empire.exceptions.InternalException;
 import org.apache.empire.exceptions.InvalidArgumentException;
+import org.apache.empire.exceptions.InvalidOperationException;
 import org.apache.empire.exceptions.NotSupportedException;
 import org.apache.empire.exceptions.UnexpectedReturnValueException;
 import org.apache.empire.jsf2.controls.TextAreaInputControl;
 import org.apache.empire.jsf2.controls.TextInputControl;
 import org.apache.empire.jsf2.impl.FacesImplementation;
 import org.apache.empire.jsf2.impl.ResourceTextResolver;
+import org.apache.empire.jsf2.pages.Page;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -161,6 +166,7 @@ public abstract class WebApplication
     public void onViewNotFound(final FacesContext fc, final HttpServletRequest req)
     {   // View not Found Error
         log.warn("No view found for request to '{}'. Use FacesUtils.redirectDirectly() to redirect to valid view.", req.getRequestURI());
+        redirectDirectly(fc, StringUtils.EMPTY);
     }
 
     /**
@@ -251,6 +257,70 @@ public abstract class WebApplication
         return getTextResolver(getContextLocale(ctx));
     }
 
+    /**
+     * Returns a FacesMessage 
+     * @param ctx the FacesContext
+     * @param severity the message severity
+     * @param msg the message or message key
+     * @param params the message params
+     * @return the FacesMessage or null to ignore
+     */
+    public FacesMessage getFacesMessage(FacesContext ctx, Severity severity, String msg, Object... params)
+    {
+        TextResolver resolver = getTextResolver(getContextLocale(ctx));
+        msg = resolver.resolveText(msg);
+        if (params.length>0)
+        {   // translate params
+            for (int i=0; i<params.length; i++)
+                if (params[i] instanceof String)
+                    params[i] = resolver.resolveText((String)params[i]);
+                else if ((params[i] instanceof Integer) || (params[i] instanceof Long))
+                    params[i] = String.valueOf(params[i]); // avoid group separator
+            // format
+            msg = MessageFormat.format(msg, params);
+        }
+        return new FacesMessage(severity, msg, null);
+    }
+    
+    /**
+     * Returns a FacesMessage for an Exception 
+     * @param ctx the FacesContext
+     * @param errorContext the error context (optional)
+     * @param t the exception
+     * @return the FacesMessage or null to ignore
+     */
+    public FacesMessage getFacesErrorMessage(FacesContext ctx, String errorContext, Throwable t)
+    {
+        // Wrap exception if necessary
+        EmpireException e = (t instanceof EmpireException) ? ((EmpireException)t) : new InternalException(t); 
+        // Get Message
+        TextResolver tr = getTextResolver(ctx);
+        String msg = tr.getExceptionMessage(e);
+        String msgDetail = extractErrorMessageDetail(errorContext, t, 3);
+        // create Faces Message
+        return new FacesMessage(FacesMessage.SEVERITY_ERROR, msg, msgDetail);
+    }
+    
+    protected String extractErrorMessageDetail(String errorContext, Throwable e, int stackTraceElements)
+    {
+        StringBuilder b = new StringBuilder();
+        if (errorContext!=null)
+        {   // Append context String
+            b.append(errorContext);
+            b.append(": ");
+        }
+        b.append(e.toString());
+        b.append("\r\nat:");
+        StackTraceElement[] stack = e.getStackTrace();
+        int len = (stack.length>stackTraceElements) ? stackTraceElements : stack.length; 
+        for (int i=0; i<len; i++)
+        {
+            b.append(stack[i].toString());
+            b.append("\r\n");
+        }
+        return b.toString();
+    }
+
     /**
      * checks if the current context contains an error
      * @param fc the FacesContext
@@ -270,47 +340,80 @@ public abstract class WebApplication
         }
         return false;
     }
-
+    
     /**
-     * Logs an exception as Error with additional information 
-     * @param message the message to log
-     * @param e the underlying exception
+     * Redirects to another page or url
+     * @param fc the FacesContext
+     * @param url the target url
      */
-    public void logException(Logger logger, String message, Throwable e)
+    public void redirectDirectly(final FacesContext fc, String url)
     {
-        if (logger==null)
-            logger=log;
-        // View Exceptions
-        if ((e instanceof ViewExpiredException)) // || (e instanceof ViewNotFoundException)) -- MyFaces only
-        {   // Session expired
-            logger.warn(message+": "+e.getMessage());
-            return;
+        try
+        {   // check params
+            if (fc==null)
+                throw new InvalidArgumentException("fc", fc);
+            if (fc.getResponseComplete())
+                throw new InvalidOperationException("Unable to redirect. Response is already complete!");
+            if (url==null)
+                url = StringUtils.EMPTY; // redirect to root
+            // Prepend Context-Path
+            ExternalContext ec = fc.getExternalContext(); 
+            String ctxPath = ec.getRequestContextPath();
+            if (url.indexOf("://")>0)
+            {   // Should not contain the context-path
+                if (url.startsWith("http") && url.indexOf(ctxPath)>0)
+                    log.warn("Redirect url \"{}\" contains protokoll and context-path. Please remove.", url);
+                else
+                    log.info("Redirecting to absolute url {}", url);
+            }
+            else if (!url.startsWith(ctxPath))
+            {   // assemble url
+                String sep = (url.length()>0 && url.charAt(0)!='/' ? "/" : null);
+                url = StringUtils.concat(ctxPath, sep, url);
+                // relative
+                log.debug("Redirecting to relative url {}", url);
+            }
+            else
+                log.warn("Redirect url \"{}\" already contains context-path. Please remove.", url);
+            // redirectDirectly
+            ec.redirect(url);
+            fc.responseComplete();
         }
-        // Get Session Info
-        String addlInfo;
-        try {
-            FacesContext fc = FacesContext.getCurrentInstance();
-            ExternalContext ec = (fc!=null ? fc.getExternalContext() : null);
-            String sessionInfo = (ec!=null ? getExceptionSessionInfo(ec.getSessionMap()) : "[NoSession]");
-            addlInfo = StringUtils.coalesce(sessionInfo, "[NoSessionInfo]");
-        } catch(Exception ex) {
-            addlInfo = "[Error:"+ex.getMessage()+"] ";
+        catch (IOException e)
+        {   // What can we do now?
+            log.error("Failed to redirect to {}", url, e);
         }
-        // Log now
-        if (e instanceof UserLevelException)
-            logger.info(StringUtils.concat(addlInfo, e.getClass().getSimpleName(), ": ", e.getMessage()));
-        else
-            logger.error(StringUtils.concat(addlInfo, message), e);                
     }
     
     /**
-     * Override this to provide additonal session info for an exception
-     * @param sessionMap
-     * @return the additional session info
+     * Handles an exeption, that could not be handled on the page level
+     * The application should redirect to the error page.
+     * @param context the faces context
+     * @param source the page from which the exception originated
+     * @param e the Exception to handle
      */
-    protected String getExceptionSessionInfo(Map<String, Object> sessionMap)
+    public void handleException(FacesContext context, Page source, Throwable e)
     {
-        return null;
+        // log source
+        String origin = (source!=null ? source.getPageDefinition().getPageBeanName() : "[Unknown]");
+        log.error("Fatal error of type {} from \"{}\": {}: {}", e.getClass().getName(), origin, e.getMessage());
+
+        // For page errors, give the ExceptionHandler a chance to handle
+        if (source!=null)
+        {
+            // Queue event 
+            ExceptionQueuedEventContext event = new ExceptionQueuedEventContext(context, e, null, context.getCurrentPhaseId());
+            event.getAttributes().put (ExceptionQueuedEventContext.IN_BEFORE_PHASE_KEY, Boolean.TRUE);
+            context.getApplication().publishEvent (context, ExceptionQueuedEvent.class, event);
+            
+            // Handle Exception
+            context.getExceptionHandler().handle(); 
+            if (context.getResponseComplete())
+                return;
+        }
+            
+        // If all has failed, redirect to ContextPath (root)
+        redirectDirectly(context, StringUtils.EMPTY);
     }
     
     /**
@@ -420,7 +523,7 @@ public abstract class WebApplication
     /**
      * returns the default input control type for a given data Type
      * @see org.apache.empire.jsf2.controls.InputControlManager
-     * @param dataType
+     * @param dataType the data type
      * @return an Input Cnotrol type
      */
     public String getDefaultControlType(DataType dataType)
