@@ -39,7 +39,7 @@ public class MenuListTag extends UIOutput // implements NamingContainer
     
     protected final TagEncodingHelper helper = TagEncodingHelperFactory.create(this, TagStyleClass.MENU.get());
     
-    private enum Properties
+    private enum MenuProperty
     {
         currentId,
         currentClass,
@@ -48,7 +48,6 @@ public class MenuListTag extends UIOutput // implements NamingContainer
         expandedClass,
         itemWrapTag,
         defaultItemClass;
-        // autoItemId
     }
     
     protected String currentId = null; 
@@ -58,7 +57,6 @@ public class MenuListTag extends UIOutput // implements NamingContainer
     protected String expandedClass = null;
     protected String itemWrapTag = null;
     protected String defaultItemClass = null; // e.g. "level{}"
-    // protected Boolean autoItemId = null;
     protected int level = 0;
     
     private MenuListTag parentMenu = null; 
@@ -86,7 +84,7 @@ public class MenuListTag extends UIOutput // implements NamingContainer
         helper.writeComponentId(writer);
         
         // Style class and style
-        helper.writeAttribute(writer, "class", getStyleClass());
+        helper.writeAttribute(writer, "class", getMenuStyleClass());
         helper.writeAttribute(writer, "style", helper.getTagAttributeString("style"));
     }
 
@@ -117,78 +115,78 @@ public class MenuListTag extends UIOutput // implements NamingContainer
     
     protected void initMenuAttributes(FacesContext context)
     {        
-        currentId        = helper.getTagAttributeString(Properties.currentId.name()); 
-        currentClass     = helper.getTagAttributeString(Properties.currentClass.name()); 
-        disabledClass    = helper.getTagAttributeString(Properties.disabledClass.name()); 
-        parentClass      = helper.getTagAttributeString(Properties.parentClass.name());
-        expandedClass    = helper.getTagAttributeString(Properties.expandedClass.name());
-        itemWrapTag      = helper.getTagAttributeString(Properties.itemWrapTag.name());
-        defaultItemClass = helper.getTagAttributeString(Properties.defaultItemClass.name());
+        currentId        = helper.getTagAttributeString(MenuProperty.currentId.name()); 
+        currentClass     = helper.getTagAttributeString(MenuProperty.currentClass.name()); 
+        parentClass      = helper.getTagAttributeString(MenuProperty.parentClass.name());
+        disabledClass    = helper.getTagAttributeString(MenuProperty.disabledClass.name()); 
+        expandedClass    = helper.getTagAttributeString(MenuProperty.expandedClass.name());
+        itemWrapTag      = helper.getTagAttributeString(MenuProperty.itemWrapTag.name());
+        defaultItemClass = helper.getTagAttributeString(MenuProperty.defaultItemClass.name());
 
         // find parent
         if (parentMenu==null)
             parentMenu = findParentMenu();
         if (parentMenu==null)
-            return;
-        
-        if (currentId==null)
-            currentId = parentMenu.getCurrentId();
-        if (currentClass==null)
-            currentClass = parentMenu.getCurrentClass();  
-        if (disabledClass==null)
-            disabledClass = parentMenu.getDisabledClass();
-        if (parentClass==null)
-            parentClass = parentMenu.getParentClass();
-        if (expandedClass==null)
-            expandedClass = parentMenu.getExpandedClass();
-        if (itemWrapTag==null)
-            itemWrapTag = parentMenu.getItemWrapTag();
-        if (defaultItemClass==null)
-            defaultItemClass = parentMenu.defaultItemClass;
-        
-        // Copy parent Info
-        level = parentMenu.level + 1;
+        {   // the root menu
+            if (currentClass==null)
+                currentClass = "current";  
+            if (parentClass==null)
+                parentClass = "parent";
+            if (disabledClass==null)
+                disabledClass = "parent";
+            if (expandedClass==null)
+                expandedClass = "expanded";
+            // level
+            level = 0;
+        }
+        else
+        {   // copy from parent
+            if (currentId==null)
+                currentId = parentMenu.getCurrentItemId();
+            if (currentClass==null)
+                currentClass = parentMenu.getCurrentItemClass();  
+            if (parentClass==null)
+                parentClass = parentMenu.getParentItemClass();
+            if (disabledClass==null)
+                disabledClass = parentMenu.getItemDisabledClass();
+            if (expandedClass==null)
+                expandedClass = parentMenu.getItemExpandedClass();
+            if (itemWrapTag==null)
+                itemWrapTag = parentMenu.getItemWrapperTagName();
+            if (defaultItemClass==null)
+                defaultItemClass = parentMenu.defaultItemClass;
+            // increase level
+            level = parentMenu.level + 1;
+        }
     }
     
-    public String getCurrentId()
+    public String getCurrentItemId()
     {
-        if (currentId==null)
-            currentId= StringUtils.toString(getStateHelper().get(Properties.currentId));
         return currentId;
     }
 
-    public String getCurrentClass()
+    public String getCurrentItemClass()
     {
-        if (currentClass==null)
-            currentClass= StringUtils.toString(getStateHelper().get(Properties.currentClass));
         return currentClass;
     }
 
-    public String getDisabledClass()
+    public String getParentItemClass()
     {
-        if (disabledClass==null)
-            disabledClass= StringUtils.toString(getStateHelper().get(Properties.disabledClass));
-        return disabledClass;
-    }
-
-    public String getParentClass()
-    {
-        if (parentClass==null)
-            parentClass= StringUtils.toString(getStateHelper().get(Properties.parentClass));
         return parentClass;
     }
 
-    public String getExpandedClass()
+    public String getItemDisabledClass()
+    {
+        return disabledClass;
+    }
+
+    public String getItemExpandedClass()
     {
-        if (expandedClass==null)
-            expandedClass= StringUtils.toString(getStateHelper().get(Properties.expandedClass));
         return expandedClass;
     }
 
-    public String getItemWrapTag()
+    public String getItemWrapperTagName()
     {
-        if (itemWrapTag==null)
-            itemWrapTag= StringUtils.toString(getStateHelper().get(Properties.itemWrapTag));
         return itemWrapTag;
     }
     
@@ -197,7 +195,7 @@ public class MenuListTag extends UIOutput // implements NamingContainer
         return level;
     }
     
-    protected String getStyleClass()
+    public String getMenuStyleClass()
     {
         String styleClass = helper.getTagAttributeString(InputControl.CSS_STYLE_CLASS);
         if (styleClass!=null && styleClass.indexOf("{}")>0)
@@ -214,74 +212,96 @@ public class MenuListTag extends UIOutput // implements NamingContainer
         // return default
         return defaultItemClass;
     }
-    
-    /*
-    public Boolean isAutoItemId()
-    {
-        if (this.autoItemId == null) {
-            this.autoItemId = (Boolean)getStateHelper().get(Properties.autoItemId);
-            if (this.autoItemId==null) {
-                if (parentMenu==null)
-                    parentMenu = findParentMenu();
-                if (parentMenu!=null)
-                    return parentMenu.isAutoItemId();
-            }
-        }
-        return this.autoItemId;
-    }
-    */
 
-    /* setters */
+    /* 
+     * Getters and setter with getStateHelper()
+     * Removed with EMPIREDB-441 on 2024-10-12
+     * 
+    
+    public String getCurrentItemId()
+    {
+        if (currentId==null)
+            currentId= StringUtils.toString(getStateHelper().get(MenuProperty.currentId));
+        return currentId;
+    }
+
+    public String getCurrentItemClass()
+    {
+        if (currentClass==null)
+            currentClass= StringUtils.toString(getStateHelper().get(MenuProperty.currentClass));
+        return currentClass;
+    }
+
+    public String getParentItemClass()
+    {
+        if (parentClass==null)
+            parentClass= StringUtils.toString(getStateHelper().get(MenuProperty.parentClass));
+        return parentClass;
+    }
+
+    public String getItemDisabledClass()
+    {
+        if (disabledClass==null)
+            disabledClass= StringUtils.toString(getStateHelper().get(MenuProperty.disabledClass));
+        return disabledClass;
+    }
+
+    public String getItemExpandedClass()
+    {
+        if (expandedClass==null)
+            expandedClass= StringUtils.toString(getStateHelper().get(MenuProperty.expandedClass));
+        return expandedClass;
+    }
+
+    public String getItemWrapperTagName()
+    {
+        if (itemWrapTag==null)
+            itemWrapTag= StringUtils.toString(getStateHelper().get(MenuProperty.itemWrapTag));
+        return itemWrapTag;
+    }
     
     public void setCurrentId(String currentId)
     {
         this.currentId = currentId;
         // save
-        getStateHelper().put(Properties.currentId, currentId);
+        getStateHelper().put(MenuProperty.currentId, currentId);
     }
 
     public void setCurrentClass(String currentClass)
     {
         this.currentClass = currentClass;
         // save
-        getStateHelper().put(Properties.currentClass, currentClass);
+        getStateHelper().put(MenuProperty.currentClass, currentClass);
     }
 
     public void setDisabledClass(String disabledClass)
     {
         this.disabledClass = disabledClass;
         // save
-        getStateHelper().put(Properties.disabledClass, disabledClass);
+        getStateHelper().put(MenuProperty.disabledClass, disabledClass);
     }
 
     public void setParentClass(String parentClass)
     {
         this.parentClass = parentClass;
         // save
-        getStateHelper().put(Properties.parentClass, parentClass);
+        getStateHelper().put(MenuProperty.parentClass, parentClass);
     }
 
     public void setExpandedClass(String expandedClass)
     {
         this.expandedClass = expandedClass;
         // save
-        getStateHelper().put(Properties.expandedClass, expandedClass);
+        getStateHelper().put(MenuProperty.expandedClass, expandedClass);
     }
 
     public void setItemWrapTag(String itemWrapTag)
     {
         this.itemWrapTag = itemWrapTag;
         // save
-        getStateHelper().put(Properties.itemWrapTag, itemWrapTag);
-    }
-
-    /*
-    public void setAutoItemId(Boolean autoItemId)
-    {
-        this.autoItemId = autoItemId;
-        // save
-        getStateHelper().put(Properties.autoItemId, this.autoItemId);
+        getStateHelper().put(MenuProperty.itemWrapTag, itemWrapTag);
     }
+    
     */
 
     /*
