@@ -43,6 +43,7 @@ import org.apache.empire.exceptions.InvalidArgumentException;
 import org.apache.empire.exceptions.ItemNotFoundException;
 import org.apache.empire.exceptions.UnexpectedReturnValueException;
 import org.apache.empire.jsf2.app.TextResolver;
+import org.apache.empire.jsf2.utils.HtmlUtils;
 import org.apache.empire.jsf2.utils.TagStyleClass;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -241,6 +242,22 @@ public abstract class InputControl
         }
     }
     
+    /**
+     * Returns the formatted value
+     * Do not override this function, but override formatValue(Object, ValueInfo) instead.   
+     * @param value the value to format
+     * @param vi the valueInfo
+     * @param escapeHtml when true the value will be escaped for Html
+     * @return the formatted value
+     */
+    public String formatValue(Object value, ValueInfo vi, boolean escapeHtml)
+    {
+        String s = formatValue(value, vi);
+        if (escapeHtml && s!=null && s.length()!=0)
+            s = HtmlUtils.getInstance().escapeText(s);
+        return s;
+    }
+    
     /**
      * Renders the control value with a surrounding HTML tag, if a tagName is supplied
      * @param comp the JSF component
@@ -283,6 +300,7 @@ public abstract class InputControl
     
     /**
      * Renders the control value without a surrounding tag (Text only)
+     * @param value the value to render
      * @param vi the value info
      * @param writer the output writer
      * @throws IOException from ResponseWriter
@@ -290,7 +308,9 @@ public abstract class InputControl
     public void renderValue(Object value, ValueInfo vi, ResponseWriter writer)
         throws IOException
     {
-        String text = formatValue(value, vi);
+        // check if html needs to be escaped
+        boolean escapeHtml = !hasFormatOption(vi, "noescape"); 
+        String text = formatValue(value, vi, escapeHtml);
         writer.append((StringUtils.isEmpty(text) ? HTML_EXPR_NBSP : text));
     }
 
@@ -338,6 +358,8 @@ public abstract class InputControl
             return; /* May want to override this */
         // Clear submitted value
         clearSubmittedValue(input);
+        // Clear local values
+        clearLocalValues(fc, input);
     }
     
     public Object getInputValue(UIComponent comp, InputInfo ii, boolean submitted)
@@ -429,7 +451,7 @@ public abstract class InputControl
         {   // Null
             styleClass += " eValNull";
         }
-        else if (dataType.isNumeric())
+        else if (dataType.isNumeric() && value instanceof Number)
         {   // Check negative
             if (ObjectUtils.getLong(value)<0)
                 styleClass += " eValNeg";
@@ -462,6 +484,11 @@ public abstract class InputControl
         throw new ItemNotFoundException("UIInput");
     }
     
+    protected boolean isInputValueExpressionEnabled()
+    {
+        return InputControlManager.isInputValueExpressionEnabled();
+    }
+    
     protected void setInputValue(UIInput input, InputInfo ii)
     {
         // Restore submitted value
@@ -487,13 +514,13 @@ public abstract class InputControl
         /* -------------------------------------- */
 
         // Assign value
-        Object value = ii.getValue(false);
+        boolean evalExpression = !isInputValueExpressionEnabled();
+        Object value = ii.getValue(evalExpression);
         if (value instanceof ValueExpression)
-        {
-            input.setValue(null);
-            input.setLocalValueSet(false);
-            input.setValueExpression("value", (ValueExpression) value);
-
+        {   // set value expression
+            ValueExpression current = input.getValueExpression("value");
+            if (current!=value)
+                setInputValueExpression(input, (ValueExpression)value, ii);
             // Object check = ((ValueExpression)value).getValue(FacesContext.getCurrentInstance().getELContext());
             // log.info("Expression value is {}.", check);
         }
@@ -505,6 +532,13 @@ public abstract class InputControl
             addRemoveValueNullStyle(input, ObjectUtils.isEmpty(value));
         }
     }
+    
+    protected void setInputValueExpression(UIInput input, ValueExpression value, InputInfo ii)
+    {
+        input.setValue(null);
+        input.setLocalValueSet(false);
+        input.setValueExpression("value", value);
+    }
 
     protected void clearSubmittedValue(UIInput input)
     {
@@ -516,6 +550,42 @@ public abstract class InputControl
         if (reqMap.containsKey(clientId))
             reqMap.remove(clientId);
     }
+    
+    protected void clearLocalValues(FacesContext context, UIComponent comp)
+    {
+        // UIInput
+        if (comp instanceof UIInput)
+        {   // Check LocalValue set 
+            UIInput input = (UIInput)comp; 
+            if (input.isValid() && input.isLocalValueSet())
+            {   // Check ValueExpression
+                // @see: UIInput:updateModel(FacesContext context)
+                ValueExpression expression = input.getValueExpression("value");
+                if (expression != null)
+                {   // Reset localValue if ValueExpression is set
+                    input.resetValue();
+                }
+            }
+            // we're done here
+            return;
+        }
+        // clearLocalValues of all facets and children of this UIComponent
+        if (comp.getFacetCount() > 0)
+        {
+            for (UIComponent facet : comp.getFacets().values())
+            {
+                clearLocalValues(context, facet);
+            }
+        }
+        // clear children
+        if (comp.getChildCount() > 0)
+        {
+            for (UIComponent child : comp.getChildren())
+            {
+                clearLocalValues(context, child);
+            }
+        }
+    }
 
     protected Object formatInputValue(Object value, InputInfo ii)
     {
@@ -748,18 +818,14 @@ public abstract class InputControl
         if (value == null)
             value = getFormatOption(vi, InputControl.FORMAT_NULL, InputControl.FORMAT_NULL_ATTRIBUTE);
         // Convert to String
-        String s = StringUtils.toString(value, StringUtils.EMPTY);
-        if (hasFormatOption(vi, "noencode"))
-            return s;
-        // Encode Html
-        return escapeHTML(s);
+        return StringUtils.toString(value, StringUtils.EMPTY);
     }
 
     /**
      * Returns the display text for an option
-     * @param options
-     * @param value
-     * @param vi
+     * @param options the options
+     * @param value the value
+     * @param vi the value info
      * @return the display text or null if the option value could not be resolved
      */
     protected String getOptionText(Options options, Object value, ValueInfo vi)
@@ -779,41 +845,6 @@ public abstract class InputControl
         return null; 
     }
 
-    /**
-     * Returns the value formated as a string
-     * This is a shortcut for formatString(vi.getValue(), vi)
-     * Derived classes may override formatString
-     
-    protected final String formatValue(ValueInfo vi)
-    {
-        // boolean hasError = ((vi instanceof InputInfo) && !((InputInfo)vi).isValid()); 
-        return formatValue(vi.getValue(true), vi);
-    }
-    */
-
-    /**
-     * escapes a String for html
-     * 
-     * @param text
-     * @return the escaped html String
-     */
-    protected String escapeHTML(String text)
-    {
-        if (text==null || text.length()==0)
-            return text;
-        // &amp;
-        if (text.indexOf('&')>=0)
-            text = StringUtils.replaceAll(text, "&", "&amp;");
-        // &lt;
-        if (text.indexOf('<')>=0)
-            text = StringUtils.replaceAll(text, "<", "&lt;");
-        // &gt;
-        if (text.indexOf('>')>=0)
-            text = StringUtils.replaceAll(text, ">", "&gt;");
-        // done
-        return text;
-    }
-
     /**
      * checks if a particular formating option has been specified.
      * 
