@@ -19,8 +19,11 @@
 package org.apache.empire.jsf2.controls;
 
 import java.io.IOException;
+import java.math.BigDecimal;
+import java.math.RoundingMode;
 import java.sql.Timestamp;
 import java.text.DateFormat;
+import java.text.DecimalFormat;
 import java.text.NumberFormat;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
@@ -28,13 +31,17 @@ import java.util.Date;
 import java.util.List;
 import java.util.Locale;
 
+import javax.el.ValueExpression;
+import javax.faces.component.StateHolder;
 import javax.faces.component.UIComponent;
 import javax.faces.component.UIInput;
+import javax.faces.component.html.HtmlForm;
 import javax.faces.component.html.HtmlInputText;
 import javax.faces.component.html.HtmlOutputText;
 import javax.faces.component.html.HtmlPanelGroup;
 import javax.faces.context.FacesContext;
 import javax.faces.context.ResponseWriter;
+import javax.faces.convert.Converter;
 import javax.faces.event.PhaseId;
 
 import org.apache.empire.commons.ObjectUtils;
@@ -42,8 +49,12 @@ import org.apache.empire.commons.Options;
 import org.apache.empire.commons.StringUtils;
 import org.apache.empire.data.Column;
 import org.apache.empire.data.DataType;
+import org.apache.empire.db.exceptions.FieldIllegalValueException;
 import org.apache.empire.exceptions.InvalidArgumentException;
+import org.apache.empire.exceptions.InvalidValueException;
 import org.apache.empire.exceptions.UnexpectedReturnValueException;
+import org.apache.empire.jsf2.components.ControlTag;
+import org.apache.empire.jsf2.components.InputTag;
 import org.apache.empire.jsf2.utils.TagStyleClass;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -62,6 +73,105 @@ public class TextInputControl extends InputControl
 
     public static final String  FRACTION_DIGITS       = "fraction-digits:";
     
+    /**
+     * NumberInputConverter
+     * Formats a decimal value based on the NumberFormat
+     */
+    public static class NumberInputConverter implements Converter, StateHolder
+    {
+        private NumberFormat nf;
+        private boolean trans = false;
+        
+        /*
+         * Must have a default constructor!
+         */
+        public NumberInputConverter()
+        {
+            this.nf = null;
+        }
+        
+        public NumberInputConverter(NumberFormat nf)
+        {
+            this.nf = nf;
+        }
+
+        @Override
+        public Object saveState(FacesContext context)
+        {
+            return nf;
+        }
+
+        @Override
+        public void restoreState(FacesContext context, Object state)
+        {
+            this.nf = (NumberFormat)state;
+        }
+
+        @Override
+        public boolean isTransient()
+        {
+            return trans;
+        }
+
+        @Override
+        public void setTransient(boolean newTransientValue)
+        {
+            this.trans = newTransientValue;
+        }
+        
+        @Override
+        public String getAsString(FacesContext context, UIComponent component, Object value)
+        {
+            if (ObjectUtils.isEmpty(value))
+                return StringUtils.EMPTY;
+            // Check number
+            if (!(value instanceof Number)) {
+                log.error("getAsString: \"{}\" is not a number!", value.getClass().getName());
+                return value.toString();
+            }
+            // format
+            return (nf!=null ?  nf.format(value) : value.toString());
+        }
+
+        @Override
+        public Object getAsObject(FacesContext context, UIComponent component, String value)
+        {
+            if (ObjectUtils.isEmpty(value))
+                return null;
+            try
+            {   // parse
+                if (nf==null)
+                    return new BigDecimal(value);
+                // parse now
+                Number number = nf.parse(value);
+                if (number instanceof BigDecimal)
+                {   // Round to scale
+                    int scale = nf.getMaximumFractionDigits();
+                    number = ((BigDecimal)number).setScale(scale, RoundingMode.HALF_UP);
+                }
+                return number;
+            }
+            catch (ParseException e)
+            {   // find column
+                UIComponent inputComp = component.getParent();
+                while (inputComp!=null) {
+                    // set the tag
+                    if ((inputComp instanceof InputTag) || (inputComp instanceof ControlTag))
+                    {   // Found an InputTag or ControlTag
+                        Object column = inputComp.getAttributes().get("column");
+                        if (column instanceof Column)
+                            throw new FieldIllegalValueException((Column)column, value);
+                    }
+                    inputComp = inputComp.getParent();
+                    if (inputComp instanceof HtmlForm)
+                        break;
+                }
+                // Just throw an InvalidValueException
+                throw new InvalidValueException(value);
+            }
+        }
+    }
+    
     private Class<? extends HtmlInputText> inputComponentClass;
 
     public TextInputControl(String name, Class<? extends HtmlInputText> inputComponentClass)
@@ -145,10 +255,9 @@ public class TextInputControl extends InputControl
 
         // disabled
         DisabledType disabled = ii.getDisabled();
-        if (disabled!=null)
-        {   input.setReadonly((disabled==DisabledType.READONLY));
-            input.setDisabled((disabled==DisabledType.DISABLED));
-        }
+        input.setReadonly((disabled==DisabledType.READONLY));
+        input.setDisabled((disabled==DisabledType.DISABLED));
+        
         // set value
         if (phaseId==PhaseId.RENDER_RESPONSE)
         {   // style
@@ -185,6 +294,29 @@ public class TextInputControl extends InputControl
             super.setInputStyleClass(input, cssStyleClass);
     }
 
+    @Override
+    protected void setInputValueExpression(UIInput input, ValueExpression value, InputInfo ii)
+    {
+        super.setInputValueExpression(input, value, ii);
+        // establish converter for decimal
+        DataType dataType = ii.getColumn().getDataType();
+        if (dataType.isNumeric())
+        {   // get number format
+            NumberFormat nf;
+            if (dataType == DataType.INTEGER || dataType == DataType.AUTOINC)
+                nf = NumberFormat.getIntegerInstance(ii.getLocale()); // Integer only
+            else {
+                // Decimal or Float
+                nf = getNumberFormat(dataType, ii, ii.getColumn());
+                // ParseBigDecimal
+                if (nf instanceof DecimalFormat)
+                    ((DecimalFormat)nf).setParseBigDecimal((dataType==DataType.DECIMAL));
+            }
+            // create converter
+            input.setConverter(new NumberInputConverter(nf));
+        }
+    }
+    
     // ------- parsing -------
 
     @Override
@@ -204,12 +336,17 @@ public class TextInputControl extends InputControl
         }
         // Check other types
         if (type == DataType.INTEGER)
-        {
-        	return parseInteger(value, ii.getLocale());
+        {   // get IntegerFormat and parse
+            NumberFormat nf = NumberFormat.getIntegerInstance(ii.getLocale());
+        	return parseNumber(value, nf);
         }
         if (type == DataType.DECIMAL || type == DataType.FLOAT)
-        {
-        	return parseDecimal(value, ii.getLocale());
+        {   // get number format
+            NumberFormat nf = getNumberFormat(type, ii, column);
+            if (nf instanceof DecimalFormat)
+                ((DecimalFormat)nf).setParseBigDecimal((type==DataType.DECIMAL));
+            // parse
+            return parseNumber(value, nf);
         }
         if (type == DataType.DATE || type == DataType.DATETIME || type == DataType.TIMESTAMP)
         {
@@ -255,11 +392,6 @@ public class TextInputControl extends InputControl
 
     @Override
     protected String formatValue(Object value, ValueInfo vi)
-    {
-        return formatValue(value, vi, true);
-    }
-
-    protected String formatValue(Object value, ValueInfo vi, boolean escapeHTML)
     {
         // Lookup and return text
         Options options = vi.getOptions();
@@ -283,13 +415,7 @@ public class TextInputControl extends InputControl
         DataType dataType = getValueType(value, (column != null) ? column.getDataType() : DataType.UNKNOWN);
         if (dataType.isText() || dataType == DataType.UNKNOWN)
         { // String
-            String s = String.valueOf(value);
-            if (hasFormatOption(vi, "noencode"))
-                return s;
-            // Encoded text
-            if (escapeHTML)
-                s = escapeHTML(s);
-            return s;
+            return String.valueOf(value);
         }
         if (dataType == DataType.INTEGER || dataType == DataType.AUTOINC)
         { // Integer
@@ -315,10 +441,6 @@ public class TextInputControl extends InputControl
          *  }
          */
         // Convert to String
-        if (escapeHTML)
-        {
-            return escapeHTML(String.valueOf(value));
-        }
         return String.valueOf(value);
     }
 
@@ -348,7 +470,7 @@ public class TextInputControl extends InputControl
         if (options != null && !options.isEmpty())
             return value;
         // Format
-        return formatValue(value, ii, false);
+        return formatValue(value, ii);
     }
 
     // ------- render -------
@@ -525,7 +647,7 @@ public class TextInputControl extends InputControl
         String type = StringUtils.valueOf(column.getAttribute(Column.COLATTR_NUMBER_TYPE));
         boolean isInteger = "Integer".equalsIgnoreCase(type);
         NumberFormat nf = (isInteger) ? NumberFormat.getIntegerInstance(locale)
-                                      : NumberFormat.getNumberInstance(locale); 
+                                      : NumberFormat.getNumberInstance(locale);
         // Groups Separator?
         Object groupSep = column.getAttribute(Column.COLATTR_NUMBER_GROUPSEP);
         nf.setGroupingUsed(groupSep != null && ObjectUtils.getBoolean(groupSep));
@@ -566,6 +688,12 @@ public class TextInputControl extends InputControl
             nf.setMinimumFractionDigits(minFactionDigits);
             nf.setMaximumFractionDigits(maxFactionDigits);
         }
+        else if (!isInteger && dataType==DataType.DECIMAL) 
+        {   // Detect from column
+            int length = (int)column.getSize();
+            int maxFactionDigits = (int)(column.getSize()*10)-(length*10);
+            nf.setMaximumFractionDigits(maxFactionDigits);
+        }
         // IntegerDigits (left-padding)
         Object intDigits = column.getAttribute(Column.COLATTR_INTEGER_DIGITS);
         if (intDigits != null) {
@@ -649,36 +777,28 @@ public class TextInputControl extends InputControl
 
     // ------- value parsing -------
 
-    protected Object parseInteger(String value, Locale locale)
+    protected Object parseNumber(String value, NumberFormat nf)
     {
-        NumberFormat nf = NumberFormat.getIntegerInstance();
-        // Parse String
         try
+        {   // Parse Number
+            Number number = nf.parse(value);
+            if (number instanceof BigDecimal)
+            {   // Round to scale
+                int scale = nf.getMaximumFractionDigits();
+                number = ((BigDecimal)number).setScale(scale, RoundingMode.HALF_UP);
+            }
+            return number;
+        }
+        catch (ParseException pe)
         {
-            return nf.parseObject(value);
-        } catch (ParseException pe) {
-            throw new NumberFormatException("Not a number: " + value + " Exception: " + pe.toString());
-        }        
-    }
-
-    protected Object parseDecimal(String value, Locale locale)
-    {
-        NumberFormat nf = NumberFormat.getNumberInstance(locale);
-        // Parse String
-        try
-        {
-            return nf.parseObject(value);
-        } catch (ParseException pe) {
             throw new NumberFormatException("Not a number: " + value + " Exception: " + pe.toString());
         }
     }
 
     protected Object parseDate(String s, DateFormat df)
     {
-        // Try to convert
         try
-        {
-            // Parse Date
+        {   // Parse Date
             df.setLenient(true);
             return df.parseObject(s);
         } catch (ParseException pe) {
