@@ -18,7 +18,6 @@
  */
 package org.apache.empire.jsf2.pageelements;
 
-import java.util.HashSet;
 import java.util.List;
 import java.util.Set;
 
@@ -36,12 +35,11 @@ import org.apache.empire.db.DBReader;
 import org.apache.empire.db.DBRecordData;
 import org.apache.empire.db.DBRowSet;
 import org.apache.empire.db.expr.order.DBOrderByExpr;
-import org.apache.empire.dbms.DBMSHandler;
 import org.apache.empire.dbms.DBMSFeature;
+import org.apache.empire.dbms.DBMSHandler;
 import org.apache.empire.exceptions.InternalException;
 import org.apache.empire.exceptions.InvalidArgumentException;
-import org.apache.empire.exceptions.UnspecifiedErrorException;
-import org.apache.empire.exceptions.NotSupportedException;
+import org.apache.empire.exceptions.InvalidOperationException;
 import org.apache.empire.exceptions.ObjectNotValidException;
 import org.apache.empire.exceptions.UnexpectedReturnValueException;
 import org.apache.empire.jsf2.app.FacesUtils;
@@ -57,20 +55,22 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
     // *Deprecated* private static final long serialVersionUID = 1L;
 
     private static final Logger log                  = LoggerFactory.getLogger(BeanListPageElement.class);
-    
+
     public static final String  NO_RESULT_ATTRIBUTE  = "noQueryResult";
 
     private ListTableInfo       listTableInfo        = null;
 
-    protected DBRowSet          rowset;
+    protected final DBContext   context;
+
+    protected final DBRowSet    rowset;
 
     protected Column            defaultSortColumn;
 
     protected boolean           defaultSortAscending = true;
 
     protected DBOrderByExpr     secondarySortOrder   = null;
-    
-    protected int               maxItemCount = 1000;
+
+    protected int               maxItemCount         = 1000;
     
     /**
      * Extended ListTableInfo
@@ -96,11 +96,12 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
      * Constructor for creating a BeanListPageElement
      * @param page the page element
      * @param beanClass the bean class
+     * @param context the database context
      * @param rowset required Table or View
      * @param defaultSortColumn sort column that must belong to rowset
      * @param propertyName the property name which is used to get and retrieve session information
      */
-    public BeanListPageElement(Page page, Class<T> beanClass, DBRowSet rowset, DBColumn defaultSortColumn, String propertyName)
+    public BeanListPageElement(Page page, Class<T> beanClass, DBContext context, DBRowSet rowset, DBColumn defaultSortColumn, String propertyName)
     {
         super(page, beanClass, propertyName);
         // Check
@@ -116,6 +117,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
         }
         // Set Bean Class and more
         this.rowset = rowset;
+        this.context = context;
         this.defaultSortColumn = defaultSortColumn;
     }
 
@@ -123,34 +125,37 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
      * Overload that requires a default sort order to be provided
      * @param page the page element
      * @param beanClass the bean class
-     * @param defaultSortColumn
+     * @param context the database context
+     * @param defaultSortColumn the default sort column
      * @param propertyName the property name which is used to get and retrieve session information
      */
-    public BeanListPageElement(Page page, Class<T> beanClass, DBColumn defaultSortColumn, String propertyName)
+    public BeanListPageElement(Page page, Class<T> beanClass, DBContext context, DBColumn defaultSortColumn, String propertyName)
     {
-        this(page, beanClass, defaultSortColumn.getRowSet(), defaultSortColumn, propertyName);
+        this(page, beanClass, context, defaultSortColumn.getRowSet(), defaultSortColumn, propertyName);
     }
 
     /**
      * Overload that requires a default sort order to be provided
      * @param page the page element
      * @param beanClass the bean class
-     * @param defaultSortColumn
+     * @param context the database context
+     * @param defaultSortColumn the default sort column
      */
-    public BeanListPageElement(Page page, Class<T> beanClass, DBColumn defaultSortColumn)
+    public BeanListPageElement(Page page, Class<T> beanClass, DBContext context, DBColumn defaultSortColumn)
     {
-        this(page, beanClass, defaultSortColumn.getRowSet(), defaultSortColumn, getDefaultPropertyName(defaultSortColumn.getRowSet()));
+        this(page, beanClass, context, defaultSortColumn.getRowSet(), defaultSortColumn, getDefaultPropertyName(defaultSortColumn.getRowSet()));
     }
 
     /**
      * Overload that requires a default sort order to be provided
      * @param page the page element
      * @param beanClass the bean class
-     * @param rowset required Table or View
+     * @param context the database context
+     * @param rowSet required Table or View
      */
-    public BeanListPageElement(Page page, Class<T> beanClass, DBRowSet rowSet)
+    public BeanListPageElement(Page page, Class<T> beanClass, DBContext context, DBRowSet rowSet)
     {
-        this(page, beanClass, rowSet, null, getDefaultPropertyName(rowSet));
+        this(page, beanClass, context, rowSet, null, getDefaultPropertyName(rowSet));
     }
     
     @Override
@@ -242,13 +247,13 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
     /**
      * Init list items with pagination
      * 
-     * @param queryCmd
-     * @param pageSize
+     * @param queryCmd the query command
+     * @param countCmd the count command
+     * @param pageSize the page size
      */
     public void initItems(DBCommand queryCmd, DBCommand countCmd, int pageSize)
     {
         clearItems();
-        DBContext context = getDBContext(rowset);
         // Init List Table Info
         BeanListTableInfo lti = (BeanListTableInfo) getTableInfo();
         lti.setQueryCmd(queryCmd);
@@ -285,7 +290,8 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
     /**
      * Init list items without pagination
      * 
-     * @param queryCmd
+     * @param queryCmd the query commmand
+     * @param pageSize the page size
      */
     public final void initItems(DBCommand queryCmd, int pageSize)
     {
@@ -296,7 +302,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
     /**
      * Init list items without pagination
      * 
-     * @param queryCmd
+     * @param queryCmd the query commmand
      */
     public final void initItems(DBCommand queryCmd)
     {
@@ -305,7 +311,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
 
     /**
      * Returns true if (and only if) items are valid but empty
-     * @return
+     * @return true if reault is empty
      */
     public boolean isResultEmpty()
     {
@@ -315,13 +321,13 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
 
     /**
      * loads all visible list items from the database
-     * @param initScrollbar
+     * @param initialize flag whether to initialize or update the pagination / scrollbar
      */
-    protected void loadItems(boolean initScrollbar)
+    protected void loadItems(boolean initialize)
     {
         // DBReader
         BeanListTableInfo lti = (BeanListTableInfo) getTableInfo();
-        DBReader r = new DBReader(getDBContext(rowset));
+        DBReader r = new DBReader(context);
         try
         { // Check command
             DBCommand queryCmd = lti.getQueryCmd();
@@ -381,8 +387,9 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
             items = r.getBeanList(beanClass, maxItems);
             if (items == null)
                 throw new UnexpectedReturnValueException(items, "DBReader.getBeanList");
-            generateIdParams(rowset, items);
-            assignSelectionMap(items);
+            
+            // Init items
+            initListItems(items);
 
             // set position at session object
             if (loadPageFromPosition)
@@ -412,7 +419,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
             // Pagination
             if (lti.isAllowPagination())
             { // Scrollbar   
-                if (initScrollbar)
+                if (initialize)
                     initScrollbar();
                 else
                     updateScrollbar();
@@ -423,13 +430,13 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
     /**
      * set order by for db queries
      * 
-     * @param cmd
+     * @param cmd the command
      */
     protected void setOrderBy(DBCommand cmd)
     {
         cmd.clearOrderBy();
         String sortColumnName = getTableInfo().getSortColumnName();
-        boolean sortAscending = getTableInfo().getSortAscending();
+        boolean sortAscending = getTableInfo().isSortAscending();
 
         // Find Column
         DBColumnExpr sortColumn = rowset.getColumn(sortColumnName);
@@ -468,13 +475,14 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
         }
     }
     
-    /* Scrollbar relacted functions */
+    /* Scrollbar related functions */
 
     /**
-     * addJavascriptCall to initScrollbar
+     * Allows to initialize the scrollbar (if present)
      */
     public void initScrollbar()
     {
+        /*
         ListTableInfo lti = getTableInfo();
         // init Scrollbar on page
         int max = lti.getItemCount() - lti.getPageSize();
@@ -495,13 +503,15 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
         b.append(String.valueOf(lti.getPageSize()));
         b.append(");");
         getPage().addJavascriptCall(b.toString());
+        */
     }
 
     /**
-     * addJavascriptCall to updateScrollbar
+     * Allows to update the scrollbar (if present)
      */
     public void updateScrollbar()
     {
+        /*
         int pos = getScrollbarPosition();
         StringBuilder b = new StringBuilder();
         b.append("updateScrollbar('");
@@ -510,6 +520,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
         b.append(String.valueOf(pos));
         b.append(");");
         getPage().addJavascriptCall(b.toString());
+        */
     }
 
     public int getScrollbarPosition()
@@ -536,45 +547,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
         lti.setPosition((pos > 0 ? pos : 0));
     }
 
-    // @Override
-    public Set<Object[]> getSelectedItemKeys()
-    {
-        if (selectedItems == null)
-            return null;
-        // Get the set
-        Set<Object[]> items = new HashSet<Object[]>(selectedItems.size());
-        for (String idParam : selectedItems)
-        {
-            Object[] key = getParameterMap().getKey(rowset, idParam);
-            if (key == null)
-            {
-                log.warn("Object does not exist in ParameterMap!");
-                continue;
-            }
-            items.add(key);
-        }
-        return items;
-    }
-
-    public void setSelectedItems(Set<Object[]> items)
-    {
-        if (selectedItems == null)
-            throw new NotSupportedException(this, "setSelectedItems");
-        // Get the set
-        selectedItems = new SelectionSet(items.size());
-        for (Object[] key : items)
-        {
-            if (key == null || key.length == 0)
-            {
-                log.warn("Cannot select Null-Object.");
-                continue;
-            }
-            String idParam = getParameterMap().put(rowset, key);
-            selectedItems.add(idParam);
-        }
-    }
-
-    protected void generateIdParams(DBRowSet rowset, List<?> items)
+    protected void initListItems(List<?> items)
     {
         Column[] keyCols = rowset.getKeyColumns();
         if (keyCols == null)
@@ -582,15 +555,23 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
         // generate all
         for (Object item : items)
         {
-            if (!(item instanceof ParameterizedItem))
-                continue;
-            // set the idParam
-            Object[] key = getItemKey(keyCols, item);
-            String idparam = getParameterMap().put(rowset, key);
-            ((ParameterizedItem)item).setIdParam(idparam);
+            initListItem(item, getItemKey(keyCols, item));
         }
     }
 
+    protected void initListItem(Object item, Object[] key)
+    {
+        if ((item instanceof ParameterizedItem))
+        {   // set the idParam
+            String idparam = getParameterMap().put(rowset, key);
+            ((ParameterizedItem)item).setIdParam(idparam);
+        }
+        if (item instanceof SelectableItem)
+        {
+            ((SelectableItem) item).initSelect(key, this.selectedItemKeys);
+        }
+    }
+    
     protected Object[] getItemKey(Column[] cols, Object item)
     {
         Object[] key = new Object[cols.length];
@@ -604,7 +585,7 @@ public class BeanListPageElement<T> extends ListPageElement<T> implements ListIt
             { // Bean Property Name
                 String propName = cols[i].getBeanPropertyName();
                 if (propName == null || propName.length() == 0)
-                    throw new UnspecifiedErrorException("Invalid Bean Property Name");
+                    throw new InvalidOperationException("Invalid Bean Property Name");
                 // Get Property value
                 try
                 {
