@@ -35,9 +35,12 @@ public enum TagStyleClass
     INPUT_MOD("eInpModified"),
     INPUT_HINT("eInputHint"),
     INPUT_TYPE_PREFIX("eInpType"),
+    INPUT_WRAPPER("eWrapInp"),
+    VALUE_WRAPPER("eWrapVal"),
     CONTROL("eControl"),
     CONTROL_LABEL("eCtlLabel"),
     CONTROL_INPUT("eCtlInput"),
+    CONTROL_PLACEHOLDER("eCtlPlaceholder"),
     FORM_GRID("eFormGrid"),
     SELECT("eSelect"),
     RADIO("eCtlRadio"),
@@ -46,6 +49,7 @@ public enum TagStyleClass
     MENU("eMenuList"),
     UNIT("eUnit"),
     TAB_VIEW("eTabView"),
+    TAB_BLIND("eTabBlind"),
     TAB_BAR("eTabBar"),
     TAB_BAR_PADDING("eTabBarEmpty"),
     TAB_PANEL("eTabPanel"),
@@ -78,8 +82,10 @@ public enum TagStyleClass
 
     private static final int find(String styleClasses, String styleClassName, int fromIdx)
     {
+        if (styleClasses==null)
+            return -1;
         // find
-        int idx = (styleClasses!=null ? styleClasses.indexOf(styleClassName, fromIdx) : -1);
+        int idx = styleClasses.indexOf(styleClassName, fromIdx);
         if (idx<0)
             return -1;
         // starts with space?
@@ -133,6 +139,9 @@ public enum TagStyleClass
     
     public static final String removeFrom(String styleClasses, String styleClassName)
     {
+        // Check name
+        if (styleClassName==null)
+            return styleClasses;
         // find
         int idx = find(styleClasses, styleClassName, 0);
         if (idx<0)
