@@ -45,6 +45,8 @@ public final class ClassUtils
 {
     // Logger
     private static final Logger log = LoggerFactory.getLogger(ClassUtils.class);
+
+    public static final Object[] EMPTY_ARGS = new Object[0];
     
     /*
      * ClassUtils contains static methods only
@@ -344,9 +346,9 @@ public final class ClassUtils
         }
         catch (NoSuchFieldException e)
         {   // try superclass
-            clazz = clazz.getSuperclass();
-            if (clazz!=null && !clazz.equals(java.lang.Object.class)) {
-                setPrivateFieldValue(clazz, object, property, value);
+            Class<?> superClass = clazz.getSuperclass();
+            if (superClass!=null && !superClass.equals(java.lang.Object.class)) {
+                setPrivateFieldValue(superClass, object, property, value);
                 return;
             }
             // not found
@@ -377,6 +379,7 @@ public final class ClassUtils
     
     /**
      * Creates a new Object instance
+     * @param <T> the class type
      * @param typeClass the class of the object to instantiate
      * @return the instance
      */
@@ -391,7 +394,15 @@ public final class ClassUtils
         {
             throw new InvalidOperationException(typeClass.getName()+" has no default constructor.");
         }
-        catch (InstantiationException | IllegalAccessException | IllegalArgumentException | SecurityException | InvocationTargetException e)
+        catch (InvocationTargetException e)
+        {   // Unwrap exception and re-throw the cause exception 
+            Throwable cause = e.getCause();
+            if (cause instanceof EmpireException)
+                throw (EmpireException)cause;
+            // wrap    
+            throw new InternalException(cause);
+        }
+        catch (InstantiationException | IllegalAccessException | IllegalArgumentException | SecurityException e)
         {
             throw new InternalException(e);
         }
@@ -399,9 +410,10 @@ public final class ClassUtils
     
     /**
      * Creates a new Object instance with a single parameter constructor
+     * @param <T> the type of the new object
      * @param typeClass the type of the object to instantiate
      * @param paramClass the type of the constructor parameter
-     * @param typeClass the value of the constructor parameter 
+     * @param paramValue the value of the constructor parameter 
      * @return the instance
      */
     public static <T> T newInstance(Class<T> typeClass, Class<?> paramClass, Object paramValue)
@@ -412,7 +424,9 @@ public final class ClassUtils
 
     /**
      * Creates a new Object instance
+     * @param <T> the type of the new object
      * @param typeConstructor the constructor of the object to instantiate
+     * @param params the constructor params
      * @return the instance
      */
     public static <T> T newInstance(Constructor<T> typeConstructor, Object... params)
@@ -424,7 +438,15 @@ public final class ClassUtils
             // create
             return typeConstructor.newInstance(params);
         }
-        catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException e)
+        catch (InvocationTargetException e)
+        {   // Unwrap exception and re-throw the cause exception 
+            Throwable cause = e.getCause();
+            if (cause instanceof EmpireException)
+                throw (EmpireException)cause;
+            // wrap    
+            throw new InternalException(cause);
+        }
+        catch (InstantiationException | IllegalAccessException | IllegalArgumentException e)
         {
             throw new InternalException(e);
         }
@@ -438,6 +460,7 @@ public final class ClassUtils
      * @param parameterTypes the param types
      * @return the constructor or null
      */
+    @SuppressWarnings("unchecked")
     public static <T> Constructor<T> findMatchingConstructor(Class<T> clazz, int minParams, Class<?>... parameterTypes)
     {
         // Minimum matching params
@@ -463,7 +486,8 @@ public final class ClassUtils
         
         // get all public constructors 
         Constructor<?>[] ctors = clazz.getConstructors();
-
+        Constructor<T> foundCtor = null;
+        
         // Search through all valid lengths
         for (int pLen = paramSize; pLen >= minParams; pLen--)
         {   
@@ -508,14 +532,17 @@ public final class ClassUtils
                         } catch (SecurityException se) {
                             // ignore 
                         }
-                        @SuppressWarnings("unchecked")
                         Constructor<T> typedCtor = (Constructor<T>) ctor;
                         return typedCtor;
                     }
+                    else
+                    {   // set as found constructor
+                        foundCtor = (Constructor<T>) ctors[i];
+                    }
                 }
             }
         }
-        return null;
+        return foundCtor;
     }
     
     /**
@@ -627,9 +654,10 @@ public final class ClassUtils
 
     /**
      * Invoke a simple method (without parameters) on an object using reflection
-     * @param clazz
+     * @param clazz the class on which to invoke the method
      * @param object the object instance on which to invoke the method
      * @param methodName the name of the method to invoke 
+     * @param makeAccessible flag whether or not to make the method accessible (eg. if private)
      * @return the return value of the method
      */
     public static Object invokeSimpleMethod(Class<?> clazz, Object object, String methodName, boolean makeAccessible)
@@ -665,6 +693,67 @@ public final class ClassUtils
         return invokeSimpleMethod(object.getClass(), object, methodName, true);
     }
 
+    /**
+     * Invoke a simple method (without parameters) on an object
+     * @param object the object on which to invoke the method
+     * @param method the method
+     * @return the return value
+     */
+    public static Object invokeSimpleMethod(Object object, Method method)
+    {
+        try
+        {   // check params
+            if (object==null || method==null)
+                throw new InvalidArgumentException("object|method", null);
+            // invoke
+            return method.invoke(object, EMPTY_ARGS);
+        }
+        catch (InvocationTargetException e)
+        {   // Unwrap exception and re-throw the cause exception 
+            Throwable cause = e.getCause();
+            if (cause instanceof EmpireException)
+                throw (EmpireException)cause;
+            // wrap    
+            throw new InternalException(cause);
+        }
+        catch (IllegalAccessException | IllegalArgumentException e)
+        {
+            throw new NotSupportedException(object, method.getName(), e); 
+        } 
+    }
+
+    /**
+     * Invoke a method with parameters on an object
+     * @param object the object on which to invoke the method
+     * @param method the method
+     * @param params the method params
+     * @return the return value
+     */
+    public static Object invokeMethod(Object object, Method method, Object... params)
+    {
+        try
+        {   // check params
+            if (object==null || method==null)
+                throw new InvalidArgumentException("object|method", null);
+            if (method.getParameterCount()!=params.length)
+                throw new InvalidArgumentException("params.length", params.length);
+            // invoke
+            return method.invoke(object, params);
+        }
+        catch (InvocationTargetException e)
+        {   // Unwrap exception and re-throw the cause exception 
+            Throwable cause = e.getCause();
+            if (cause instanceof EmpireException)
+                throw (EmpireException)cause;
+            // wrap    
+            throw new InternalException(cause);
+        }
+        catch (IllegalAccessException | IllegalArgumentException e)
+        {
+            throw new NotSupportedException(object, method.getName(), e); 
+        } 
+    }
+    
     /**
      * Returns the JAR name that contains the implementation of a specific class
      * @param clazz the class 
