@@ -43,6 +43,12 @@ import org.w3c.dom.Element;
 public class Options extends AbstractSet<OptionEntry> implements Cloneable, Serializable
 {
 	private static final long serialVersionUID = 1L;
+	
+	@FunctionalInterface
+	public interface OptionGroupResolver 
+	{
+	    Object getGroup(OptionEntry oe);
+	}
 
     /**
      * Implements the Map interface for Options
@@ -152,7 +158,9 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
         Top, Bottom, Sort
     }
 
-    private final ArrayList<OptionEntry> list;
+    protected final ArrayList<OptionEntry> list;
+    
+    protected OptionGroupResolver optionGroupResolver;
     
     public Options()
     {   // Default constructor
@@ -208,6 +216,27 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
         return new Options(this);
     }
 
+    /**
+     * Returns the function that determines the group to which an option entry belongs.
+     * @return the group resolver function or null
+     */
+    public OptionGroupResolver getOptionGroupResolver()
+    {
+        return optionGroupResolver;
+    }
+
+    /**
+     * Sets a function that determines the group to which an option entry belongs.
+     * e.g.:
+     * options.setOptionGroupResolver((oe) -&gt; ((MyEnum)oe.getValue()).getCategory());
+     * 
+     * @param optionGroupResolver the group resolver function
+     */
+    public void setOptionGroupResolver(OptionGroupResolver optionGroupResolver)
+    {
+        this.optionGroupResolver = optionGroupResolver;
+    }
+
     protected int getIndex(Object value)
     {
         if (value instanceof OptionEntry)
@@ -281,7 +310,7 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
 
     /**
      * Returns all values as a set
-     * boolean activeOnly flag whether to return the active items only
+     * @param activeOnly flag whether to return the active items only
      * @return the value set
      */
     public Set<Object> getValues(boolean activeOnly)
@@ -380,6 +409,7 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
      * 
      * @param value the value object
      * @param text the text
+     * @param active flag whether or not the option is active
      */
     public void set(Object value, String text, Boolean active)
     {
@@ -395,7 +425,7 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
         }
         else
         {   // add new Option
-            list.add(createOptionEntry(value, text, (active!=null ? active :true )));
+            append(value, text, (active!=null ? active :true ));
         }
     }
     
@@ -416,6 +446,7 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
      * @param value the value
      * @param text the text for this value
      * @param active flag if element is active (selectable)
+     * @return returns self (this)
      */
     public final Options add(Object value, String text, boolean active)
     {
@@ -428,6 +459,7 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
      * Same as set() but allows Option building 
      * @param value the value
      * @param text the text for this value
+     * @return returns self (this)
      */
     public final Options add(Object value, String text)
     {
@@ -438,17 +470,28 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
     @Override
     public boolean add(OptionEntry option)
     {
-        if (option==null || option.getText() == null)
+        if (option==null)
             throw new InvalidArgumentException("option", option);
         // find and add or replace
         OptionEntry oe = getEntry(option.getValue());
         if (oe!=null)
             list.set(getIndex(oe), option);
         else
-            list.add(option);
+            append(option);
         return true;
     }
     
+    /**
+     * Appends an option
+     * Useful for fast loading when it is certain that there are no duplicates
+     * WARNING: Does not check if the entry already exists 
+     * @param option the option to append
+     */
+    public void append(OptionEntry option)
+    {
+        list.add(option);
+    }
+    
     /**
      * Appends an option
      * Useful for fast loading when it is certain that there are no duplicates
@@ -461,7 +504,7 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
     {
         if (text == null)
             text = StringUtils.EMPTY;
-        list.add(createOptionEntry(value, text, active));
+        append(createOptionEntry(value, text, active));
     }
 
     @Override
@@ -541,7 +584,9 @@ public class Options extends AbstractSet<OptionEntry> implements Cloneable, Seri
     }
     
     /**
-     * Returns an immutable Map for the options 
+     * Returns an immutable Map for the options
+     * @param <T> the type of the map
+     * @param type the map type 
      * @return the map of options
      */
     public <T> Map<T, String> map(Class<T> type)
