@@ -133,6 +133,7 @@ public abstract class DBColumnExpr extends DBExpr
 
     /**
      * returns an expression that renames the column with its alias name
+     * @return the rename expression
      */
     public DBColumnExpr qualified()
     {
@@ -147,7 +148,7 @@ public abstract class DBColumnExpr extends DBExpr
      * @return value of the attribute if it exists or null otherwise
      */
     @Override
-    public synchronized Object getAttribute(String name)
+    public Object getAttribute(String name)
     {
         if (attributes != null && attributes.indexOf(name)>=0)
             return attributes.get(name);
@@ -160,9 +161,10 @@ public abstract class DBColumnExpr extends DBExpr
 
     /**
      * Sets the value of a column attribute.
-     * 
+     * @param <T> the column expression type
      * @param name the attribute name
      * @param value the value of the attribute
+     * @return returns self (this)
      */
     @SuppressWarnings("unchecked")
     public synchronized <T extends DBColumnExpr> T setAttribute(String name, Object value)
@@ -180,7 +182,7 @@ public abstract class DBColumnExpr extends DBExpr
      * @return the list of options
      */
     @Override
-    public synchronized Options getOptions()
+    public Options getOptions()
     {
         if (options != null)
             return options;
@@ -193,11 +195,12 @@ public abstract class DBColumnExpr extends DBExpr
 
     /**
      * Sets the options for this column indicating all valid values.
-     * 
+     * @param <T> the column expression type
      * @param options the list of options
+     * @return returns self (this)
      */
     @SuppressWarnings("unchecked")
-    public synchronized <T extends DBColumnExpr> T setOptions(Options options)
+    public <T extends DBColumnExpr> T setOptions(Options options)
     {
         this.options = options;
         return (T)this;
@@ -217,8 +220,9 @@ public abstract class DBColumnExpr extends DBExpr
 
     /**
      * Sets the title attribute.
-     * 
+     * @param <T> the column expression type
      * @param title the column title
+     * @return returns self (this)
      */
     public <T extends DBColumnExpr> T setTitle(String title)
     { 
@@ -241,8 +245,9 @@ public abstract class DBColumnExpr extends DBExpr
 
     /**
      * Sets the controlType attribute.
-     * 
+     * @param <T> the column expression type
      * @param controlType the column control type
+     * @return returns self (this)
      */
     public final <T extends DBColumnExpr> T setControlType(String controlType)
     { 
@@ -257,54 +262,21 @@ public abstract class DBColumnExpr extends DBExpr
      * @return the name of the bean property used to get and set values 
      */
     @Override
-    public synchronized String getBeanPropertyName()
+    public String getBeanPropertyName()
     {
         if (beanPropertyName==null)
-        {   // Compute bean property name
-            String name = getName();
-            if (name==null)
-                return null; // no name provided!
-            // compute name
-            name = name.toLowerCase();        
-            String res = "";
-            int beg=0;
-            while (beg<name.length())
-            {
-                int end = name.indexOf('_', beg);
-                if (end<0)
-                    end = name.length();
-                // assemble
-                if (end>beg)
-                {
-                    if (beg==0)
-                    {   // begin with all lower cases
-                        res = name.substring(beg, end);
-                    }
-                    else
-                    {   // add word where first letter is upper case 
-                        res += name.substring(beg, beg+1).toUpperCase();
-                        if (end-beg>1)
-                        {
-                            res += name.substring(beg+1, end);
-                        }
-                    }
-                }
-                // next
-                beg = end + 1;
-            }
-            // Result
-            beanPropertyName = res;
-        }
+            beanPropertyName = StringUtils.toCamelCase(getName(), false); // Compute bean property name
         return beanPropertyName;
     }
 
     /**
      * Sets the Java bean property name for this column.
-     *
-     * @param propertyName
+     * @param <T> the column expression type
+     * @param propertyName the property name
+     * @return returns self (this)
      */
     @SuppressWarnings("unchecked")
-    public synchronized <T extends DBColumnExpr> T setBeanPropertyName(String propertyName)
+    public <T extends DBColumnExpr> T setBeanPropertyName(String propertyName)
     {
         this.beanPropertyName = propertyName; 
         return (T)this;
@@ -458,7 +430,7 @@ public abstract class DBColumnExpr extends DBExpr
 
     /**
      * Creates and returns an expression for the SQL "in" operator. 
-     * 
+     * @param <T> the value type
      * @param values the values to compare this column with
      * @return a DBCompareColExpr for the "in" operator
      */
@@ -498,7 +470,7 @@ public abstract class DBColumnExpr extends DBExpr
 
     /**
      * Creates and returns an expression for the SQL "not in" operator. 
-     *
+     * @param <T> the value type
      * @param values the values to compare this column with
      * @return a DBCompareColExpr for the "not in" operator
      */
@@ -709,8 +681,8 @@ public abstract class DBColumnExpr extends DBExpr
     /**
      * Creates a sql-expression for the nvl() or coalesce() function.
      * 
-     * @param nullValue the Object value
-     * @return the new DBFuncExpr object
+     * @param nullValue the alternative value when this expression is null
+     * @return the coalesce expression
      */
     public DBColumnExpr coalesce(Object nullValue)
     {
@@ -962,7 +934,7 @@ public abstract class DBColumnExpr extends DBExpr
     {
         return new DBConcatFuncExpr(this, concatExprs);
     }
-
+            
     /**
      * concatenates a list of expressions to the current column 
      * @param separator a string to insert between each of the expressions
@@ -973,6 +945,20 @@ public abstract class DBColumnExpr extends DBExpr
     {
         return new DBConcatFuncExpr(this, separator, concatExprs);
     }
+
+    /**
+     * Puts a value or expression before the current expression 
+     * @param value the expressions to prepend
+     * @return the combined value
+     */
+    public DBColumnExpr prepend(Object value)
+    {
+        String opertor  = (getDataType()==DataType.UNKNOWN ? "" 
+                        : (getDataType().isText() ? getDatabase().getDbms().getSQLPhrase(DBSqlPhrase.SQL_CONCAT_EXPR) 
+                        : "+"));
+        String template = StringUtils.concat("{0}",opertor,"?");
+        return new DBFuncExpr(this, template, new Object[] { value }, false, getDataType());
+    }
     
     /*
      * Numeric functions
@@ -1116,6 +1102,7 @@ public abstract class DBColumnExpr extends DBExpr
     /**
      * Creates and returns string aggregation expression
      * @param separator the separator between string
+     * @param orderBy the order by expression
      * @return the new DBFuncExpr object
      */
     public DBColumnExpr stringAgg(String separator, DBOrderByExpr orderBy)
