@@ -22,13 +22,14 @@ import java.lang.ref.WeakReference;
 import java.math.BigDecimal;
 import java.math.RoundingMode;
 import java.sql.Timestamp;
-import java.text.ParseException;
 import java.time.LocalDate;
 import java.time.LocalDateTime;
 import java.time.LocalTime;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Collections;
 import java.util.Date;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.List;
@@ -36,6 +37,8 @@ import java.util.Map;
 import java.util.Set;
 
 import org.apache.empire.commons.ObjectUtils;
+import org.apache.empire.commons.Options;
+import org.apache.empire.commons.StringUtils;
 import org.apache.empire.data.Column;
 import org.apache.empire.data.DataType;
 import org.apache.empire.db.DBDDLGenerator.DDLActionType;
@@ -46,16 +49,20 @@ import org.apache.empire.db.exceptions.FieldNotNullException;
 import org.apache.empire.db.exceptions.FieldValueException;
 import org.apache.empire.db.exceptions.FieldValueOutOfRangeException;
 import org.apache.empire.db.exceptions.FieldValueTooLongException;
+import org.apache.empire.db.expr.column.DBCaseExpr;
+import org.apache.empire.db.expr.column.DBCaseMapExpr;
 import org.apache.empire.db.expr.column.DBCaseWhenExpr;
 import org.apache.empire.db.expr.column.DBValueExpr;
 import org.apache.empire.db.expr.compare.DBCompareExpr;
 import org.apache.empire.dbms.DBMSHandler;
 import org.apache.empire.dbms.DBSqlPhrase;
 import org.apache.empire.exceptions.InvalidArgumentException;
+import org.apache.empire.exceptions.InvalidOperationException;
 import org.apache.empire.exceptions.ItemExistsException;
 import org.apache.empire.exceptions.NotSupportedException;
+import org.apache.empire.exceptions.ObjectNotValidException;
 import org.apache.empire.exceptions.PropertyReadOnlyException;
-import org.apache.empire.exceptions.UnspecifiedErrorException;
+import org.apache.empire.exceptions.ValueConversionException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -66,7 +73,6 @@ import org.slf4j.LoggerFactory;
  * It provides access to the various database objects such as tables, views and relations.
  * <P>
  * It also provides methods to execute DQL and DML SQL-commands.
- * <P>
  *
  */
 public abstract class DBDatabase extends DBObject
@@ -108,6 +114,8 @@ public abstract class DBDatabase extends DBObject
     
     /** 
      * find a database by id
+     * @param dbIdent the database id to look for
+     * @return the database or null if not found
      */
     public static DBDatabase findByIdentifier(String dbIdent)
     {
@@ -124,17 +132,19 @@ public abstract class DBDatabase extends DBObject
     }
     
     /** 
-     * find a database by id
+     * find a database by its Java type
+     * @param clazz the database class to look for
+     * @return the database or null if not found
      */
-    public static DBDatabase findByClass(Class<? extends DBDatabase> cls)
+    public static DBDatabase findByClass(Class<? extends DBDatabase> clazz)
     {
         for (WeakReference<DBDatabase> ref : databaseMap.values())
         {   // find database by class
             DBDatabase db = ref.get();
-            if (db!=null && cls.isInstance(db))
+            if (db!=null && clazz.isInstance(db))
                 return db;
         }
-        log.warn("Database of class {} not found!", cls.getSimpleName());
+        log.warn("Database of class {} not found!", clazz.getSimpleName());
         return null;
     }
 
@@ -147,6 +157,8 @@ public abstract class DBDatabase extends DBObject
     protected final List<DBTable>    tables    = new ArrayList<DBTable>();
     protected final List<DBRelation> relations = new ArrayList<DBRelation>();
     protected final List<DBView>     views     = new ArrayList<DBView>();
+    // map of all Rowsets by alias
+    protected final Map<String, DBRowSet> rowsetAliasMap = new HashMap<String, DBRowSet>();
     
     protected DBMSHandler dbms    = null;
     
@@ -203,7 +215,7 @@ public abstract class DBDatabase extends DBObject
     public synchronized void discard()
     {
         if (isOpen())
-            throw new UnspecifiedErrorException("Database is open. Discard not possible.");
+            throw new InvalidOperationException("Database is open. Discard not possible.");
         // unregister
         databaseMap.remove(this.instanceId);
         this.instanceId = null;
@@ -215,7 +227,7 @@ public abstract class DBDatabase extends DBObject
     
     /**
      * registers the database in the global list of databases
-     * @param dbid
+     * @param dbid the database id
      */
     protected synchronized void register(String dbid)
     {
@@ -295,8 +307,8 @@ public abstract class DBDatabase extends DBObject
 
     /**
      * Returns the DBMS Handler for this database.
-     * 
-     * @return Returns the DBMS Handler for this database
+     * @param <T> the DBMSHandler type
+     * @return returns the DBMS Handler for this database
      */
     @SuppressWarnings("unchecked")
     public <T extends DBMSHandler> T getDbms()
@@ -333,6 +345,7 @@ public abstract class DBDatabase extends DBObject
     /**
      * Checks if the database exists
      * The implementation depends on the DBMSHandler
+     * @param context the database context
      * @return true if the database exists or false otherwise 
      */
     public boolean checkExists(DBContext context)
@@ -408,7 +421,7 @@ public abstract class DBDatabase extends DBObject
             // Set dbms
             if (this.dbms!=null && this.dbms!=ddlHandler && ddlHandler!=null)
             {   // The database belongs to a different dbms
-                throw new UnspecifiedErrorException("The database is attached to a different dbms.");
+                throw new InvalidOperationException("The database is attached to a different dbms.");
             }
             // Temporarily change dbms
             if (this.dbms== null)
@@ -424,7 +437,7 @@ public abstract class DBDatabase extends DBObject
 
     /**
      * Override this to change or add DDL commands
-     * @param script
+     * @param script the script on which to add the DDL commands 
      */
     protected void generateDDLScript(DBSQLScript script)
     {
@@ -538,7 +551,7 @@ public abstract class DBDatabase extends DBObject
     
     /**
      * Returns the java class type for a given dataType
-     * @param expr the column expression for witch to return the java type
+     * @param expr the column expression for which to return the java type
      * @return return the java class used for storing values of this dataType 
      */
     public Class<?> getColumnJavaType(DBColumnExpr expr)
@@ -549,7 +562,7 @@ public abstract class DBDatabase extends DBObject
                 return Long.class;
             case INTEGER:
             {   // Check Integer size
-                DBColumn source = expr.getSourceColumn();
+                DBColumn source = expr.getUpdateColumn();
                 int size = (source!=null ? (int)source.getSize() : DBTable.INT_SIZE_BIG);
                 if (size<=DBTable.INT_SIZE_SMALL)
                     return Short.class;
@@ -638,6 +651,17 @@ public abstract class DBDatabase extends DBObject
         return new DBValueExpr(this, Long.valueOf(value), DataType.INTEGER);
     }
 
+    /**
+     * Creates and returns a value object for the given string value.
+     * 
+     * @param value the String value
+     * @return the new DBValueExpr object
+     */
+    public DBValueExpr getValueExpr(BigDecimal value)
+    {
+        return new DBValueExpr(this, value, DataType.DECIMAL);
+    }
+
     /**
      * Creates and returns a value object for the given scalar value.
      * 
@@ -647,6 +671,8 @@ public abstract class DBDatabase extends DBObject
      */
     public DBValueExpr getValueExpr(Object value, DataType dataType)
     {
+        if (value instanceof DBValueExpr)
+            return ((DBValueExpr)value); // No nesting
         return new DBValueExpr(this, value, dataType);
     }
 
@@ -681,17 +707,23 @@ public abstract class DBDatabase extends DBObject
     { // find column by name
         if (table == null || table.db != this)
             throw new InvalidArgumentException("table", table);
-        if (tables.contains(table)==true)
-            throw new ItemExistsException(table.getName());
         // Check for second instances
         DBTable existing = getTable(table.getName()); 
+        if (existing==table)
+            return; // already there
         if (existing!=null)
         {   // Check classes
-            if (existing.getClass().equals(table.getClass()))
-                return; // Ignore other instances 
+            if (isSameRowSet(existing, table))
+            {   // Add instance to alias map
+                addRowsetToAliasMap(table);
+                // Ignore other instances
+                return;  
+            }
             // Table exists with different class
             throw new ItemExistsException(table.getName());
         }
+        // Add instance to alias map
+        addRowsetToAliasMap(table);
         // add now
         tables.add(table);
     }
@@ -707,6 +739,8 @@ public abstract class DBDatabase extends DBObject
         // remove
         if (tables.contains(table))
             tables.remove(table);
+        // Remove from RowSet map
+        removeRowsetFromAliasMap(table);
     }
 
     /**
@@ -750,10 +784,31 @@ public abstract class DBDatabase extends DBObject
         return rset;
     }
 
+    /**
+     * Finds a RowSet object by the alias name.
+     *
+     * @param alias the alias of the desired table
+     * @return the located DBTable object
+     */
+    public DBRowSet getRowSetByAlias(String alias)
+    {   // find table by name
+        return this.rowsetAliasMap.get(alias);
+    }
+
+    /**
+     * Returns all DBRowSet instances currently created for this database
+     * @return all DBRowSet instances
+     */
+    public Collection<DBRowSet> getAllRowSets()
+    {   // find table by name
+        return this.rowsetAliasMap.values();
+    }
+
     /**
      * Adds a foreign key relation to the database.
-     * <P>
+     *
      * @param reference a reference for a source and target column pair
+     * @return the relation object
      */
     public final DBRelation addRelation(DBRelation.DBReference reference)
     {
@@ -767,10 +822,11 @@ public abstract class DBDatabase extends DBObject
     }
 
     /**
-     * Add a foreign key relation to the database.
+     * Adds a foreign key relation to the database.
      * 
      * @param ref1 a reference for a source and target column pair
      * @param ref2 a reference for a source and target column pair
+     * @return the relation object
      */
     public final DBRelation addRelation(DBRelation.DBReference ref1, DBRelation.DBReference ref2)
     {
@@ -790,6 +846,7 @@ public abstract class DBDatabase extends DBObject
      * 
      * @param name the relation name
      * @param references a list of source and target column pairs
+     * @return the relation object
      */
     public DBRelation addRelation(String name, DBRelation.DBReference... references)
     {
@@ -838,7 +895,7 @@ public abstract class DBDatabase extends DBObject
 
     /**
      * Returns the relation of a given name
-     * 
+     * @param relationName the name of the relation
      * @return db the relation of the given name
      */
     public DBRelation getRelation(String relationName)
@@ -862,9 +919,24 @@ public abstract class DBDatabase extends DBObject
     { // find column by name
         if (view == null || view.db != this)
             throw new InvalidArgumentException("view", view);
-        if (views.contains(view) == true)
+        // Check for second instances
+        DBView existing = getView(view.getName()); 
+        if (existing==view)
+            return; // already there
+        if (existing!=null)
+        {   // Check classes
+            if (isSameRowSet(existing, view))
+            {   // Add instance to alias map
+                addRowsetToAliasMap(view);
+                // Ignore other instances
+                return;  
+            }
+            // Table exists with different class
             throw new ItemExistsException(view.getName());
-        // add now
+        }
+        // Add instance to alias map
+        addRowsetToAliasMap(view);
+        // add view
         views.add(view);
     }
 
@@ -879,6 +951,8 @@ public abstract class DBDatabase extends DBObject
         // remove
         if (views.contains(view))
             views.remove(view);
+        // Remove from RowSet map
+        removeRowsetFromAliasMap(view);
     }
 
     /**
@@ -964,6 +1038,8 @@ public abstract class DBDatabase extends DBObject
             return DataType.CHAR;
         if (value instanceof byte[])
             return DataType.BLOB;
+        if (value instanceof Enum)
+            return DataType.VARCHAR; // Assume VARCHAR 
         return DataType.UNKNOWN;
     }
     
@@ -977,7 +1053,7 @@ public abstract class DBDatabase extends DBObject
      * @param value the checked to check for validity
      * @return the (possibly converted) value
      * 
-     * @throws FieldValueException
+     * @throws FieldValueException exception thrown if value is not valid
      */
     protected Object validateValue(DBTableColumn column, Object value)
     {
@@ -1035,10 +1111,10 @@ public abstract class DBDatabase extends DBObject
                 if (!(value instanceof LocalDateTime) && !(value instanceof Date) && !DBDatabase.SYSDATE.equals(value))
                 {   try {
                         // Parse Date
-                        value = ObjectUtils.toDate(value);
-                    } catch(ParseException e) {
+                        value = ObjectUtils.getDate(value);
+                    } catch (ValueConversionException e) {
                         log.info("Parsing '{}' to Date failed for column {}. Message is "+e.toString(), value, column.getName());
-                        throw new FieldIllegalValueException(column, String.valueOf(value), e);
+                        throw new FieldIllegalValueException(column, String.valueOf(value), e.getCause());
                     }
                 }    
                 break;
@@ -1046,14 +1122,12 @@ public abstract class DBDatabase extends DBObject
             case DECIMAL:
                 // check enum
                 if (value instanceof Enum<?>)
-                {   // convert enum   
-                    value = ((Enum<?>)value).ordinal();
-                }
+                    break; // Convert later...
                 // check number
                 if (!(value instanceof java.lang.Number))
                 {   try
                     {   // Convert to Decimal
-                        value = ObjectUtils.toDecimal(value);
+                        value = ObjectUtils.getValueUtils().toDecimal(value);
                         // throws NumberFormatException if not a number!
                     } catch (NumberFormatException e) {
                         log.info("Parsing '{}' to Decimal failed for column {}. Message is "+e.toString(), value, column.getName());
@@ -1068,7 +1142,7 @@ public abstract class DBDatabase extends DBObject
                 if (!(value instanceof java.lang.Number))
                 {   try
                     {   // Convert to Double
-                        value = ObjectUtils.toDouble(value);
+                        value = ObjectUtils.getValueUtils().toDouble(value);
                         // throws NumberFormatException if not a number!
                     } catch (NumberFormatException e) {
                         log.info("Parsing '{}' to Double failed for column {}. Message is "+e.toString(), value, column.getName());
@@ -1082,14 +1156,12 @@ public abstract class DBDatabase extends DBObject
             case INTEGER:
                 // check enum
                 if (value instanceof Enum<?>)
-                {   // convert enum   
-                    value = ((Enum<?>)value).ordinal();
-                }
+                    break; // Convert later...
                 // check number
                 if (!(value instanceof java.lang.Number))
                 {   try
                     {   // Convert to Long
-                        value = ObjectUtils.toLong(value);
+                        value = ObjectUtils.getValueUtils().toLong(value);
                     } catch (NumberFormatException e) {
                         log.info("Parsing '{}' to Integer failed for column {}. Message is "+e.toString(), value, column.getName());
                         throw new FieldIllegalValueException(column, String.valueOf(value), e);
@@ -1103,9 +1175,7 @@ public abstract class DBDatabase extends DBObject
             case CHAR:
                 // check enum
                 if (value instanceof Enum<?>)
-                {   // convert enum   
-                    value = ObjectUtils.getString((Enum<?>)value);
-                }
+                    break; // Convert later...
                 // check length
                 if (value.toString().length() > (int)column.getSize())
                 {
@@ -1143,7 +1213,7 @@ public abstract class DBDatabase extends DBObject
         // Check overall
         if (type==DataType.DECIMAL)
         {   // Convert to Decimal
-            BigDecimal dv = ObjectUtils.toDecimal(n);
+            BigDecimal dv = ObjectUtils.getValueUtils().toDecimal(n);
             int prec = dv.precision();
             int scale = dv.scale();
             // check precision and scale
@@ -1166,7 +1236,7 @@ public abstract class DBDatabase extends DBObject
     }
     
     /**
-     * Creates a case column expression
+     * Creates a SQL case expression
      * in the form "case when [condition] then [trueValue] else [falseValue] end"
      * This is a helper function to simplify client usage
      * @param condition the compare expression
@@ -1174,31 +1244,119 @@ public abstract class DBDatabase extends DBObject
      * @param falseValue the value to select if the condition is false
      * @return the case expression
      */
-    public DBColumnExpr caseWhen(DBCompareExpr condition, Object trueValue, Object falseValue)
+    public DBCaseExpr caseWhen(DBCompareExpr condition, Object trueValue, Object falseValue)
     {
-        DataType dataType = detectDataType((trueValue!=null ? trueValue : falseValue)); 
-        DBColumnExpr trueExpr = ((trueValue  instanceof DBColumnExpr) ? (DBColumnExpr)trueValue : this.getValueExpr(trueValue, dataType));
-        return trueExpr.when(condition, falseValue);
+        return new DBCaseWhenExpr(condition, trueValue, falseValue);
     }
     
-    
-    public DBColumnExpr caseWhen(Map<DBCompareExpr, DBColumnExpr> whenMap, DBColumnExpr elseValue)
+    /**
+     * Creates a SQL case expression
+     * in the form "case when [mapKey] then [mapValue] else [elseValue] end"
+     * @param whenMap the map with constraints
+     * @param elseValue the else expression
+     * @return the expression
+     */
+    public DBCaseExpr caseWhen(Map<DBCompareExpr, ? extends Object> whenMap, Object elseValue)
     {
-        return new DBCaseWhenExpr(whenMap, elseValue);
+        return new DBCaseWhenExpr(this, whenMap, elseValue);
     }
 
     /**
-     * Creates a case column expression that check whether a column or column expression is null
+     * Creates a SQL case expression that check whether a column or column expression is null
      * "case when [condition] is null then [trueValue] else [falseValue] end"
      * This is a helper function to simplify client usage
-     * @param expr a column or column expression
+     * @param column a column or column expression
      * @param trueValue the value to select if the condition is true
      * @param falseValue the value to select if the condition is false
      * @return an sql case expression
      */
-    public DBColumnExpr caseWhenNull(DBColumnExpr expr, Object trueValue, Object falseValue)
+    public DBCaseExpr caseWhenNull(DBColumnExpr column, Object trueValue, Object falseValue)
     {
-        return caseWhen(expr.is(null), trueValue, falseValue);
+        // return caseMap(column, null, trueValue, falseValue);
+        return caseWhen(column.is(null), trueValue, falseValue);
+    }
+    
+    /**
+     * Creates a SQL case expression
+     * in the form "case [Expr] when [mapKey] then [mapValue] else [elseValue] end"
+     * @param column the column expression which to map
+     * @param valueMap map of key value pairs
+     * @param elseValue the else expression
+     * @return the expression
+     */
+    public DBCaseExpr caseMap(DBColumnExpr column, Map< ? extends Object,  ? extends Object> valueMap, Object elseValue)
+    {
+        return new DBCaseMapExpr(column, valueMap, elseValue);
+    }
+    
+    /**
+     * Creates a SQL case expression
+     * in the form "case [Expr] when [optionValue] then [optionText] else [elseValue] end"
+     * @param column a column or column expression
+     * @param options the options to map
+     * @param elseValue the else expression
+     * @return the expression
+     */
+    public DBCaseExpr caseMap(DBColumnExpr column, Options options, Object elseValue)
+    {
+        return new DBCaseMapExpr(column, options.map(), elseValue);
+    }
+    
+    /**
+     * Creates a SQL case expression
+     * in the form "case [Expr] when [compareValue] then [trueValue] else [elseValue] end"
+     * @param column a column or column expression
+     * @param cmpValue the value to compare the column value with
+     * @param trueValue the true expression
+     * @param falseValue the false expression
+     * @return the expression
+     */
+    public DBCaseExpr caseMap(DBColumnExpr column, Object cmpValue, Object trueValue, Object falseValue)
+    {
+        return new DBCaseMapExpr(column, cmpValue, trueValue, falseValue);
     }
 
+    
+    /**
+     * Checks if two Rowsets with the same name are equal and just another instance
+     * @param first the first rowset
+     * @param second the second rowset
+     * @return true if both are the same and are just another instance
+     */
+    protected boolean isSameRowSet(DBRowSet first, DBRowSet second)
+    {
+        return first.getClass().equals(second.getClass());        
+    }
+   
+    /**
+     * adds a DBRowSet to the alias map
+     * @param rowset the rowset to add to the map
+     */
+    protected void addRowsetToAliasMap(DBRowSet rowset)
+    {
+        String alias = rowset.getAlias();
+        if (StringUtils.isEmpty(alias))
+            throw new ObjectNotValidException(rowset); // alias must be valid
+        // find existing
+        DBRowSet existing = rowsetAliasMap.get(alias);
+        if (existing==rowset)
+            return; // already there
+        if (existing!=null) {
+            log.error("Rowset alias \"{}\" must be unqiue, but has alreaedy been used for {}", alias, existing.getClass().getName());
+            throw new ItemExistsException(alias); // Alias must be unique!
+        }
+        // add to map
+        rowsetAliasMap.put(alias, rowset);
+    }
+
+    /**
+     * removes a rowset from the alias map
+     * @param rowset the rowset to remove
+     */
+    protected void removeRowsetFromAliasMap(DBRowSet rowset)
+    {
+        String alias = rowset.getAlias();
+        rowsetAliasMap.remove(alias);
+    }
+    
 }
\ No newline at end of file
