@@ -57,7 +57,16 @@ public class DBQuery extends DBRowSet
     // *Deprecated* private static final long serialVersionUID = 1L;
 
     private static AtomicInteger queryCount = new AtomicInteger(0);
-    
+    /**
+     * Automatically generates a new alias for this Object 
+     * @param prefix the alias prefix
+     * @return an alias consisting of the prefix and a unique number
+     */
+    protected String generateAlias(String prefix)
+    {
+        return prefix + String.valueOf(queryCount.incrementAndGet());
+    }
+
     /**
      * DBQueryExprColumn 
      * @author doebele
@@ -95,6 +104,7 @@ public class DBQuery extends DBRowSet
     protected final DBColumn[]      keyColumns;
     protected final DBQueryColumn[] queryColumns;
     protected final String          alias;
+    protected boolean               updateable;
 
     /**
      * Constructor initializes the query object.
@@ -107,6 +117,10 @@ public class DBQuery extends DBRowSet
     public DBQuery(DBCommandExpr cmd, DBColumn[] keyColumns, String alias)
     { // Set the column expressions
         super(cmd.getDatabase());
+        // generate alias
+        if (StringUtils.isEmpty(alias))
+            alias = generateAlias("q");
+        // set 
         this.cmdExpr = cmd;
         this.alias = alias;
         // Set Query Columns
@@ -129,6 +143,7 @@ public class DBQuery extends DBRowSet
         }
         // Set the key Column
         this.keyColumns = keyColumns;
+        this.updateable = false;
     }
 
     /**
@@ -140,7 +155,7 @@ public class DBQuery extends DBRowSet
      */
     public DBQuery(DBCommandExpr cmd, DBColumn[] keyColumns)
     {   // Set the column expressions
-        this(cmd, keyColumns, "q" + String.valueOf(queryCount.incrementAndGet()));
+        this(cmd, keyColumns, null);
     }
     
     /**
@@ -169,7 +184,7 @@ public class DBQuery extends DBRowSet
     }
 
     /**
-     * Creaes a DBQuery object from a given command object.
+     * Creates a DBQuery object from a given command object.
      * 
      * @param cmd the command object representing an SQL-Command.
      * @param alias the query alias
@@ -180,7 +195,7 @@ public class DBQuery extends DBRowSet
     }
 
     /**
-     * Creaes a DBQuery object from a given command object.
+     * Creates a DBQuery object from a given command object.
      * 
      * @param cmd the command object representing an SQL-Command.
      */
@@ -189,6 +204,17 @@ public class DBQuery extends DBRowSet
         this(cmd, (DBColumn[]) null);
     }
 
+    /**
+     * Creates a copy of an existing DBQuery object.
+     * @param other the query which to copy
+     * @param newAlias the new alias for the cloned query 
+     */
+    public DBQuery(DBQuery other, String newAlias)
+    { 
+        this(other.cmdExpr, other.keyColumns, newAlias);
+        this.updateable = other.updateable;
+    }
+
     /**
      * returns the underlying command expression
      * @return the command used for this query
@@ -223,7 +249,32 @@ public class DBQuery extends DBRowSet
     @Override
     public boolean isUpdateable()
     {
-        return (getKeyColumns()!=null);
+        return updateable;
+    }
+    
+    /**
+     * Makes the Query updateable. Queries are not updateable by default. 
+     * For a query to be updateable it must have key columns
+     * @param updateable flag whether or not the column can be updated
+     */
+    public void setUpdateable(boolean updateable)
+    {
+        if (updateable && getKeyColumns()==null)
+            throw new NotSupportedException(this, "setUpdateable");
+        // set updateable
+        this.updateable = updateable;
+    }
+
+    /**
+     * Gets the query column by Index
+     * @param iColumn the column index
+     * @return the query column of that index
+     */
+    public DBQueryColumn getQueryColumn(int iColumn)
+    {
+        if (iColumn < 0 || iColumn >= columns.size())
+            return null;
+        return queryColumns[iColumn];
     }
 
     /**
@@ -297,6 +348,17 @@ public class DBQuery extends DBRowSet
             throw new ItemNotFoundException(name);
         return col;
     }
+
+    /**
+     * This is a convenience shortcut for getQueryColumn
+     * 
+     * @param iColumn the index of the query column
+     * @return the located column
+     */
+    public DBQueryColumn column(int iColumn)
+    {
+        return getQueryColumn(iColumn);
+    }
     
     /**
      * return query key columns
@@ -349,10 +411,10 @@ public class DBQuery extends DBRowSet
      * Add rowset data
      */
     @Override
-    public void initRecord(DBRecordBase record, DBRecordData recData)
+    public void initRecord(DBRecordBase record, DBRecordData recData, boolean newRecord)
     {
         // init
-        super.initRecord(record, recData);
+        super.initRecord(record, recData, newRecord);
         // set record key as rowset data (optional)
         if (keyColumns!=null)
         {   // check
@@ -637,7 +699,8 @@ public class DBQuery extends DBRowSet
     
     /**
      * factory method for column expressions in order to allow overrides 
-     * @param expr
+     * @param expr the column expression for the query column
+     * @param index the column index
      * @return the query column
      */
     protected DBQueryColumn createQueryColumn(DBColumnExpr expr, int index)
@@ -679,17 +742,23 @@ public class DBQuery extends DBRowSet
     @Override
     public int getColumnIndex(DBColumn column)
     {
-        int index = columns.indexOf(column);
-        if (index>=0)
-            return index;
-        // find by update column
-        index=0;
-        for (DBColumn c : columns)
+        // (changed 2024-07-18 EMPIREDB-434)
+        // 1st try: compare columns
+        int count = columns.size();
+        for (int index=0; index<count; index++)
         {   // check update column
-            if ((c instanceof DBQueryExprColumn) && column.equals(c.getUpdateColumn()))
-                 return index;
-            // next
-            index++;
+            if (columns.get(index).equals(column))
+                return index;
+        }
+        // 2nd try: Match update column
+        for (int index=0; index<count; index++)
+        {   // check update column
+            DBColumn c = columns.get(index);
+            if ((c instanceof DBQueryExprColumn))
+            {   // Update columns match
+                if (column.equals(c.getUpdateColumn()))
+                    return index;
+            }
         }
         // not found
         return -1;
@@ -704,5 +773,11 @@ public class DBQuery extends DBRowSet
         // use column
         return column;
     }
+
+    @Override
+    protected DBColumn cloneColumn(DBRowSet clone, DBColumn scourceColumn)
+    {
+        throw new NotSupportedException(this, "cloneColumn");
+    }
   
 }
