@@ -112,7 +112,7 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
             String info = "Record "+record.getRowSet().getName();
             if (record.getKeyColumns()==null)
                 return info;
-            return info+":"+StringUtils.arrayToString(record.getKey(), "|");
+            return StringUtils.concat(info, ":", StringUtils.arrayToString(record.getKey()));
         }
 
         @Override
@@ -228,6 +228,7 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
     private boolean[]       modified;
     Object                  rowsetData; // Special Rowset Data (usually null)
     protected boolean       validateFieldValues;
+    protected boolean       allowReadOnlyUpdate;
     
     // Parent-Record-Map for deferred identity setting 
     private Map<DBColumn, DBRecordBase> parentRecordMap;
@@ -244,6 +245,7 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
         this.modified = null;
         this.rowsetData = null;
         this.validateFieldValues = true;
+        this.allowReadOnlyUpdate = false;
         this.parentRecordMap = null;
     }
 
@@ -259,6 +261,7 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
 
     /**
      * helper to check if the object is valid
+     * @param fieldIndex the field index
      * @throws ObjectNotValidException if the object is not valid
      */
     protected void checkValid(int fieldIndex)
@@ -316,6 +319,7 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
     
     /**
      * Returns whether or not RollbackHandling is enabled for this record
+     * @return true if rollback handling is enabled or false otherwise
      */
     public abstract boolean isRollbackHandlingEnabled(); 
 
@@ -490,7 +494,7 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
 
     /**
      * Returns true if any of the given fields was modified.
-     * 
+     * @param columns the columns to check
      * @return true if any of the given fields were modified or false otherwise
      */
     public final boolean wasAnyModified(Column... columns)
@@ -654,23 +658,15 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
             setParentRecord(getColumn(index), (DBRecordBase)value);
             return;
         }
-        // Strings special
-        if ((value instanceof String) && ((String)value).length()==0)
-            value = null;
         // Is value valid
         Object current = fields[index]; 
         if (current==ObjectUtils.NO_VALUE)
             throw new FieldValueNotFetchedException(getColumn(index));
         // convert
         DBColumn column = getColumn(index);
-        // must convert enums
-        if (value instanceof Enum<?>)
-        {   // convert enum
-            Enum<?> enumVal = ((Enum<?>)value);
-            boolean numeric = column.getDataType().isNumeric();
-            value = ObjectUtils.getEnumValue(enumVal, numeric);
-        }
         // Has Value changed?
+        if (value!= null)
+            value = ObjectUtils.convertValue(column.getDataType(), value);
         if (ObjectUtils.compareEqual(current, value))
         {   // value has not changed!
             return; 
@@ -698,6 +694,9 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
 
     /**
      * Deprecated Renamed to set(...)   
+     * @param column a DBColumn object
+     * @param value the value
+     * @return returns self (this)
      */
     @Deprecated
     public DBRecordBase setValue(Column column, Object value)
@@ -712,6 +711,7 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
      * 
      * @param column a DBColumn object
      * @param value the value
+     * @return returns self (this)
      */
     @Override
     public DBRecordBase set(Column column, Object value)
@@ -814,7 +814,7 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
 
     /**
      * Sets record values from the suppied java bean.
-     * @return true if at least one value has been set sucessfully
+     * @return true if at least one value has been set successfully
      */
     @Override
     public final int setRecordValues(Object bean)
@@ -994,7 +994,7 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
     
     /**
      * changes the state of the record
-     * @param newState
+     * @param newState the new record state
      */
     protected void changeState(State newState)
     {
@@ -1047,12 +1047,15 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
     /**
      * Checks whether the record is updateable  
      * If its read-only a RecordReadOnlyException is thrown 
-     * @throws RecordReadOnlyException
+     * @throws RecordReadOnlyException exception thrown if record cannot be updated
      */
     protected void checkUpdateable()
     {
         if (this.isReadOnly())
-            throw new RecordReadOnlyException(this);
+        {   // Check allowReadOnlyUpdate
+            if (!allowReadOnlyUpdate)
+                throw new RecordReadOnlyException(this);
+        }
     }
     
     /**
@@ -1079,6 +1082,7 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
      * 
      * @param index index of the column
      * @param value the column value
+     * @param fireChangeEvent flag whether to fire the onFieldChanged event
      */
     protected void modifyValue(int index, Object value, boolean fireChangeEvent)
     {   // Check valid
@@ -1099,6 +1103,30 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
         if (fireChangeEvent)
             onFieldChanged(index);
     }
+
+    protected void setFieldModified(int index, boolean modifiedFlag)
+    {   // Check valid
+        checkValid(index);
+        if (modified == null && !modifiedFlag)
+            return; // Nothing to do
+        // modified state array
+        if (modified == null)
+        {   modified = new boolean[fields.length];
+            for (int j = 0; j < fields.length; j++)
+                modified[j] = false;
+        }
+        // set value and modified
+        modified[index] = modifiedFlag;
+        // set record state
+        boolean recordModified = modifiedFlag;
+        for (int j = 0; j < fields.length && !recordModified; j++)
+            recordModified |= modified[j];
+        // Change record state if necessary
+        if (recordModified && state.isLess(State.Modified))
+            changeState(State.Modified);
+        else if (recordModified==false && state==State.Modified)
+            changeState(State.Valid);
+    }
     
     /**
      * Override this to do extra handling when the record changes
@@ -1114,6 +1142,7 @@ public abstract class DBRecordBase extends DBRecordData implements Record, Clone
     
     /**
      * Override this to get notified when a field value changes
+     * @param i the field index
      */
     protected void onFieldChanged(int i)
     {
