@@ -18,6 +18,7 @@
  */
 package org.apache.empire.db;
 
+import java.lang.reflect.Field;
 import java.sql.Connection;
 import java.sql.Timestamp;
 import java.util.ArrayList;
@@ -28,6 +29,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import org.apache.empire.commons.Attributes;
 import org.apache.empire.commons.ObjectUtils;
 import org.apache.empire.commons.StringUtils;
 import org.apache.empire.data.Column;
@@ -58,11 +60,11 @@ import org.apache.empire.dbms.DBMSFeature;
 import org.apache.empire.dbms.DBMSHandler;
 import org.apache.empire.dbms.DBSqlPhrase;
 import org.apache.empire.exceptions.InvalidArgumentException;
+import org.apache.empire.exceptions.InvalidOperationException;
 import org.apache.empire.exceptions.ItemNotFoundException;
 import org.apache.empire.exceptions.NotSupportedException;
 import org.apache.empire.exceptions.ObjectNotValidException;
 import org.apache.empire.exceptions.UnexpectedReturnValueException;
-import org.apache.empire.exceptions.InvalidOperationException;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -71,9 +73,6 @@ import org.slf4j.LoggerFactory;
  * This class is the base class for all the DBTable,
  * DBView and DBQuery classes this class contains all the columns of the
  * tables, views or queries
- * <P>
- * 
- *
  */
 public abstract class DBRowSet extends DBExpr implements EntityType
 {
@@ -150,6 +149,7 @@ public abstract class DBRowSet extends DBExpr implements EntityType
     protected DBColumn                 timestampColumn  = null;
     protected Map<DBColumn, DBColumn>  columnReferences = null;
     protected List<DBColumn>           columns          = new ArrayList<DBColumn>();
+    protected Attributes               attributes       = null;
 
     // associated Entity Bean Class (optional)
     protected Class<?>                 beanType         = null;
@@ -167,7 +167,7 @@ public abstract class DBRowSet extends DBExpr implements EntityType
             ((DBRecordBean)record).rowset = this;
         }
         // check param
-        if (record==null || record.getRowSet()!=this)
+        if (record==null || !isSame(record.getRowSet()))
             throw new InvalidArgumentException("record", record);
         // check valid
         if (checkValid && !record.isValid())
@@ -199,7 +199,44 @@ public abstract class DBRowSet extends DBExpr implements EntityType
      */
     public String getIdentifier()
     {
-        return db.getIdentifier()+"."+getName();
+        return StringUtils.concat(db.getIdentifier(), ".", getName());
+    }
+
+    /**
+     * Returns a named attribute for this table
+     * @param name the attribute name
+     * @return the attribute value
+     */
+    public Object getAttribute(String name)
+    {
+        return (attributes != null ? attributes.get(name) : null);
+    }
+
+    /**
+     * Returns all metadata attributes.
+     * @return set of metadata attributes
+     */
+    @SuppressWarnings("unchecked")
+    public Set<Attributes.Attribute> getAttributes()
+    {
+        return (attributes!=null ? Collections.unmodifiableSet(attributes)
+                                 : Collections.EMPTY_SET);
+    }
+
+    /**
+     * Sets the value of a attribute.
+     * @param <T> the table type
+     * @param name the attribute name
+     * @param value the value of the attribute
+     * @return returns self (this)
+     */
+    @SuppressWarnings("unchecked")
+    public synchronized <T extends DBTable> T setAttribute(String name, Object value)
+    {
+        if (attributes== null)
+            attributes = new Attributes();
+        attributes.set(name, value);
+        return (T)this;
     }
 
     /**
@@ -286,7 +323,7 @@ public abstract class DBRowSet extends DBExpr implements EntityType
     public boolean isSame(DBRowSet other)
     {
         // record
-        if (!getDatabase().equals(other.getDatabase()))
+        if (other==null || !getDatabase().equals(other.getDatabase()))
             return false;
         // compare names
         return StringUtils.compareEqual(getName(), other.getName(), true);
@@ -361,7 +398,9 @@ public abstract class DBRowSet extends DBExpr implements EntityType
     
     /**
      * sets the bean type for this rowset
+     * @param <T> the bean type
      * @param beanType the bean type for this rowset
+     * @param factory the bean factory
      */
     public <T> void setBeanType(Class<T> beanType, DBBeanListFactory<T> factory)
     {
@@ -408,6 +447,15 @@ public abstract class DBRowSet extends DBExpr implements EntityType
         return Collections.unmodifiableList(columns);
     }
 
+    /**
+     * Gets all columns of this rowset as an array
+     * @return all columns of this rowset
+     */
+    public DBColumn[] getAllColumns()
+    {
+        return ObjectUtils.listToArray(DBColumn[].class, columns);
+    }
+
     /**
      * Gets the index of a particular column expression.
      * @param column column the DBColumn to get the index for
@@ -587,7 +635,7 @@ public abstract class DBRowSet extends DBExpr implements EntityType
     /**
      * Returns the column expression at a given column index
      * Allow overrides in derived classes
-     * @param index
+     * @param index the index of the desired expression
      * @return the column expression
      */
     protected DBColumnExpr getColumnExprAt(int index)
@@ -597,6 +645,8 @@ public abstract class DBRowSet extends DBExpr implements EntityType
     
     /**
      * Creates a join expression based on a compare expression
+     * @param cmp the compare expression for the join
+     * @return the join expression
      */
     public DBCompareJoinExpr on(DBCompareExpr cmp)
     {
@@ -606,6 +656,8 @@ public abstract class DBRowSet extends DBExpr implements EntityType
     
     /**
      * Creates a cross join expression
+     * @param right the rowset which to cross join
+     * @return the cross join expression
      */
     public DBCrossJoinExpr on(DBRowSet right)
     {
@@ -671,13 +723,14 @@ public abstract class DBRowSet extends DBExpr implements EntityType
      * <P>
      * @param record the record object
      * @param recData the record data from which to initialized the record
+     * @param newRecord true if the record is new or false if it is an existing record
      */
-    public void initRecord(DBRecordBase record, DBRecordData recData)
+    public void initRecord(DBRecordBase record, DBRecordData recData, boolean newRecord)
     {
         // check param
         checkParamRecord(record, false);
         // Initialize the record
-        prepareInitRecord(record, false);
+        prepareInitRecord(record, newRecord);
         // Get Record Field Values
         Object[] fields = record.getFields();
         DBColumn[] keyColumns = getKeyColumns();
@@ -707,6 +760,17 @@ public abstract class DBRowSet extends DBExpr implements EntityType
         // Done
         completeInitRecord(record);
     }
+
+    /**
+     * Initializes a DBRecord for this rowset for an existing record
+     * @see org.apache.empire.db.DBRowSet#initRecord(DBRecordBase, DBRecordData, boolean)
+     * @param record the record object
+     * @param recData the record data from which to initialized the record
+     */
+    public final void initRecord(DBRecordBase record, DBRecordData recData)
+    {
+        initRecord(record, recData, false);
+    }
     
     /**
      * initializes the Record Default Values
@@ -742,7 +806,7 @@ public abstract class DBRowSet extends DBExpr implements EntityType
      */
     protected void prepareInitRecord(DBRecordBase record, boolean newRecord)
     {
-        if (record==null || record.getRowSet()!=this)
+        if (record==null || !isSame(record.getRowSet()))
             throw new InvalidArgumentException("rec", record);
         if (columns.size() < 1)
             throw new ObjectNotValidException(this);
@@ -782,7 +846,13 @@ public abstract class DBRowSet extends DBExpr implements EntityType
             DBColumn column = keyColumns[i];
             Object value = key[i];
             // set key column constraint
-            compareExpr = compareExpr.and(column.is(value));
+            DBCompareExpr colCmpExpr;
+            if (value instanceof DBCompareExpr)
+                colCmpExpr = ((DBCompareExpr)value);
+            else
+                colCmpExpr = column.is(value);
+            // append
+            compareExpr = compareExpr.and(colCmpExpr);
         }
         return compareExpr;
     }
@@ -955,6 +1025,7 @@ public abstract class DBRowSet extends DBExpr implements EntityType
      * If the record has been modified by another user, an error of type 
      * DBErrors.RecordUpdateFailed will be set.  
      * <P>
+     * @param <R> the record type
      * @param record the DBRecord object. contains all fields and the field properties
      */
     public <R extends DBRecordBase> void updateRecord(R record)
@@ -1069,8 +1140,6 @@ public abstract class DBRowSet extends DBExpr implements EntityType
                 { 	// Update a field
                     if (col.isReadOnly())
                         log.warn("updateRecord: Read-only column '" + col.getName() + " has been modified!");
-                    // *** unnecessary check removed 2.5.0 ***
-                    // col.validate(value);  
                     // Set the column
                     cmd.set(col.to(value));
                     setCount++;
@@ -1205,7 +1274,10 @@ public abstract class DBRowSet extends DBExpr implements EntityType
     }
     
     /**
-     *  Mabe use Prepared statements even if disabled in context 
+     *  Creates a new command object for record handing
+     *  May use Prepared statements even if disabled in context
+     *  @param context the db context
+     *  @return the command object 
      */
     protected DBCommand createRecordCommand(DBContext context)
     {
@@ -1240,6 +1312,75 @@ public abstract class DBRowSet extends DBExpr implements EntityType
     {
         record.rowsetData = rowsetData;
     }
+
+    /**
+     * Clone columns
+     * @param <T> the rowset type
+     * @param clone the cloned rowset 
+     * @throws CloneNotSupportedException exception if cloning the fields fails
+     */
+    protected <T extends DBRowSet> void initClonedFields(T clone) throws CloneNotSupportedException
+    {
+        // clone all columns
+        Class<?> colClass = columns.get(0).getClass();
+        Class<?> colBase = colClass.getSuperclass();
+        clone.columns = new ArrayList<DBColumn>();
+        Field[] fields = getClass().getFields();
+        for (int i = 0; i < columns.size(); i++)
+        {
+            DBColumn srcCol = columns.get(i);
+            DBColumn newCol = cloneColumn(clone, srcCol);
+            // Replace all references for oldCol to newCol
+            for (int j = 0; j < fields.length; j++)
+            {   // Find a class of Type DBColumn or DBTableColumn
+                Field f = fields[j];
+                Class<?> type = f.getType();
+                if (type == colClass || type == colBase)
+                {   try
+                    {   // Check if the field points to the old Value
+                        if (f.get(clone) == srcCol)
+                        {   // Check accessible
+                            if (f.isAccessible()==false) 
+                            {   // not accessible
+                                f.setAccessible(true);
+                                try {
+                                    f.set(clone, newCol);
+                                } finally {
+                                    f.setAccessible(false);
+                                }
+                            }
+                            else
+                            {   // already accessible
+                                f.set(clone, newCol);
+                            }
+                        }
+                    } catch (Exception e)  {
+                        // IllegalAccessException or IllegalArgumentException
+                        String fieldName = fields[j].getName();
+                        log.error("Failed to modify declared table field: " + fieldName + ". Reason is: " + e.toString());
+                        // throw CloneNotSupportedException
+                        CloneNotSupportedException cnse = new CloneNotSupportedException("Unable to replace field reference for field " + fieldName);
+                        cnse.initCause(e);
+                        throw cnse;
+                    }
+                }
+            }
+        }
+        // timestamp column
+        if (this.timestampColumn!=null)
+        {   // set cloned update column
+            int updateColumnIndex = getColumnIndex(timestampColumn);
+            clone.timestampColumn = clone.getColumn(updateColumnIndex);
+        }
+    }
+
+    /**
+     * Clones a RowSet column
+     * @param clone the cloned rowset
+     * @param scourceColumn the source column
+     * @return the cloned column
+     */
+    protected abstract DBColumn cloneColumn(DBRowSet clone, DBColumn scourceColumn);
     
 }
 
