@@ -27,7 +27,6 @@ import java.util.Collection;
 import java.util.Date;
 
 import org.apache.empire.commons.ObjectUtils;
-import org.apache.empire.commons.OptionEntry;
 import org.apache.empire.commons.StringUtils;
 import org.apache.empire.data.DataType;
 import org.apache.empire.dbms.DBMSHandler;
@@ -57,6 +56,7 @@ public abstract class DBSQLBuilder implements Appendable
     /**
      *  Don't use this directly
      *  Use dbms.createSQLBuilder()
+     *  @param dbms the dbms handler
      */
     protected DBSQLBuilder(DBMSHandler dbms)
     {
@@ -186,18 +186,9 @@ public abstract class DBSQLBuilder implements Appendable
         {   // it's an expression
             ((DBExpr) value).addSQL(this, context);
             return;
-        } 
-        // check option entry
-        if (value instanceof OptionEntry)
-        {   // option value
-            value = ((OptionEntry)value).getValue();
         }
-        // check enum
-        if (value instanceof Enum<?>)
-        {   // check enum
-            value = ObjectUtils.getEnumValue((Enum<?>)value, dataType.isNumeric());
-        }
-        else if (value instanceof Collection<?>)
+        // Check colletion
+        if (value instanceof Collection<?>)
         {   // collection 2 array
             value = ((Collection<?>)value).toArray();
         }
@@ -215,7 +206,10 @@ public abstract class DBSQLBuilder implements Appendable
             return;
         } 
         else
-        {   // Get Value Expression from dmbs
+        {   // Convert to database format
+            if (value!= null)
+                value = ObjectUtils.convertValue(dataType, value);
+            // append now
             appendSimpleValue(dataType, value);
         }
     }
@@ -290,8 +284,7 @@ public abstract class DBSQLBuilder implements Appendable
     protected void appendSimpleValue(DataType type, Object value)
     { 
         if (value instanceof Enum<?>)
-        {   // convert enum
-            log.warn("Enum of type {} supplied for getValueString. Converting value...", value.getClass().getName());
+        {   // If still an Enum, convert now (Lazy conversion)
             value = ObjectUtils.getEnumValue((Enum<?>)value, type.isNumeric());
         }
         if (ObjectUtils.isEmpty(value))
@@ -440,7 +433,8 @@ public abstract class DBSQLBuilder implements Appendable
     }
 
     /** 
-     * this helper function doubles up single quotes for SQL 
+     * this helper function doubles up single quotes for SQL
+     * @param value the string to escape and append
      */
     protected void escapeAndAppendLiteral(String value)
     {
