@@ -18,9 +18,8 @@
  */
 package org.apache.empire.db;
 
-// java
-import java.lang.reflect.Field;
 import java.sql.Connection;
+import java.text.MessageFormat;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
@@ -36,11 +35,11 @@ import org.apache.empire.db.exceptions.NoPrimaryKeyException;
 import org.apache.empire.db.exceptions.RecordDeleteFailedException;
 import org.apache.empire.db.exceptions.RecordUpdateFailedException;
 import org.apache.empire.exceptions.InvalidArgumentException;
+import org.apache.empire.exceptions.InvalidOperationException;
 import org.apache.empire.exceptions.ItemExistsException;
 import org.apache.empire.exceptions.NotSupportedException;
 import org.apache.empire.exceptions.ObjectNotValidException;
 import org.apache.empire.exceptions.UnexpectedReturnValueException;
-import org.apache.empire.exceptions.UnspecifiedErrorException;
 
 
 /**
@@ -59,6 +58,15 @@ public class DBTable extends DBRowSet implements Cloneable
     public static final int INT_SIZE_BIG     = 8;
 
     private static AtomicInteger tableCount          = new AtomicInteger(0);
+    /**
+     * Automatically generates a new alias for this Object 
+     * @param prefix the alias prefix
+     * @return an alias consisting of the prefix and a unique number
+     */
+    protected String generateAlias(String prefix)
+    {
+        return prefix + String.valueOf(tableCount.incrementAndGet());
+    }
 
     private final String         name;
     private String               alias;
@@ -78,8 +86,8 @@ public class DBTable extends DBRowSet implements Cloneable
     { 
         super(db);
         // generate alias
-        if (alias==null)
-            alias = "t" + String.valueOf(tableCount.incrementAndGet());
+        if (StringUtils.isEmpty(alias))
+            alias = generateAlias("t");
         // init
         this.name = name;
         this.alias = alias;
@@ -174,10 +182,13 @@ public class DBTable extends DBRowSet implements Cloneable
     {
         DBTable clone = (DBTable) super.clone();
         initClonedFields(clone);
+        // set primaryKey
+        clone.primaryKey = clonePrimaryKey(clone);
         // set new alias
-        clone.alias = "t" + String.valueOf(tableCount.incrementAndGet());
+        clone.alias = generateAlias("t");
         // done
         log.info("clone: Table " + name + " cloned! Alias old=" + alias + " new=" + clone.alias);
+        db.addTable(clone);
         return clone;
     }
     
@@ -187,13 +198,16 @@ public class DBTable extends DBRowSet implements Cloneable
         try {
             DBTable clone = (DBTable) super.clone();
             initClonedFields(clone);
+            // set primaryKey
+            clone.primaryKey = clonePrimaryKey(clone);
             // set new alias
             if (StringUtils.isEmpty(newAlias))
-                clone.alias = "t" + String.valueOf(tableCount.incrementAndGet());
+                clone.alias = generateAlias("t");
             else
                 clone.alias = newAlias;
             // done
             log.info("clone: Table " + name + " cloned! Alias old=" + alias + " new=" + clone.alias);
+            db.addTable(clone);
             return (T)clone;
         } catch (CloneNotSupportedException e) {
             // unable to clone table
@@ -202,53 +216,24 @@ public class DBTable extends DBRowSet implements Cloneable
         }
     }
     
-    protected <T extends DBTable> void initClonedFields(T clone) throws CloneNotSupportedException
+    @Override
+    protected DBColumn cloneColumn(DBRowSet clone, DBColumn sourceColumn)
     {
-        // clone all columns
-        Class<?> colClass = columns.get(0).getClass();
-        Class<?> colBase = colClass.getSuperclass();
-        clone.columns = new ArrayList<DBColumn>();
-        Field[] fields = getClass().getFields();
-        for (int i = 0; i < columns.size(); i++)
-        {
-            DBTableColumn srcCol = (DBTableColumn) columns.get(i);
-            DBTableColumn newCol = new DBTableColumn(clone, srcCol);
-            // Replace all references for oldCol to newCol
-            for (int j = 0; j < fields.length; j++)
-            {   // Find a class of Type DBColumn or DBTableColumn
-                Field f = fields[j];
-                Class<?> type = f.getType();
-                if (type == colClass || type == colBase)
-                {   try
-                    {   // Check if the field points to the old Value
-                        if (f.get(clone) == srcCol)
-                        {   // Check accessible
-                            if (f.isAccessible()==false) 
-                            {   // not accessible
-                                f.setAccessible(true);
-                                try {
-                                    f.set(clone, newCol);
-                                } finally {
-                                    f.setAccessible(false);
-                                }
-                            }
-                            else
-                            {   // already accessible
-                                f.set(clone, newCol);
-                            }
-                        }
-                    } catch (Exception e)  {
-                        // IllegalAccessException or IllegalArgumentException
-                        String fieldName = fields[j].getName();
-                        log.error("Failed to modify declared table field: " + fieldName + ". Reason is: " + e.toString());
-                        // throw CloneNotSupportedException
-                        CloneNotSupportedException cnse = new CloneNotSupportedException("Unable to replace field reference for field " + fieldName);
-                        cnse.initCause(e);
-                        throw cnse;
-                    }
-                }
-            }
-        }
+        DBTableColumn newCol = new DBTableColumn((DBTable)clone, (DBTableColumn)sourceColumn);
+        return newCol;
+    }
+
+    protected DBIndex clonePrimaryKey(DBRowSet clone)
+    {
+        if (this.primaryKey==null)
+            return null;
+        // clone columns
+        DBColumn[] source = this.primaryKey.getColumns(); 
+        DBColumn[] columns = new DBColumn[source.length];
+        for (int i=0; i<source.length; i++)
+            columns[i] = clone.getColumn(getColumnIndex(source[i]));
+        // Create clone
+        return new DBIndex(this.primaryKey.getName(), DBIndexType.PRIMARY_KEY, columns);
     }
 
     /**
@@ -274,7 +259,7 @@ public class DBTable extends DBRowSet implements Cloneable
      * @param defValue a Object object
      * @return the new column object
      */
-    protected DBTableColumn crateAndAppendColumn(String columnName, DataType type, double size, boolean required, Object defValue)
+    protected DBTableColumn createAndAppendColumn(String columnName, DataType type, double size, boolean required, Object defValue)
     {
         // Check exists
         if (getColumn(columnName)!=null)
@@ -288,18 +273,9 @@ public class DBTable extends DBRowSet implements Cloneable
             if (this.primaryKey==null)
                 this.setPrimaryKey(column);
             else
-                throw new UnspecifiedErrorException("Table "+getName()+" already has a Primary-Key! No column of type AUTOINC can be added.");
+                throw new InvalidOperationException("Table "+getName()+" already has a Primary-Key! No column of type AUTOINC can be added.");
         }
-        /*
-        // auto-set timestamp column
-        if (column.getDataType()==DataType.TIMESTAMP)
-        {   // Automatically set timestamp column
-            if (timestampColumn==null)
-                this.setTimestampColumn(column);
-            else
-                log.warn("Table {} already has a Timestamp column. DataType of column {} should be DATETIME.", getName(), column.getName());
-        }
-        */
+        // add now
         addColumn(column);
         return column;
     }
@@ -317,11 +293,11 @@ public class DBTable extends DBRowSet implements Cloneable
      */
     public final DBTableColumn addColumn(String columnName, DataType type, double size, boolean required, Object defValue)
     {
+        // Must not be a class
         if (defValue instanceof Class<?>)
-        {
-            log.warn("Column {}: a class object of type \"{}\" has been passed as default value. Please check!", columnName, ((Class<?>)defValue).getName());
-        }
-        return this.crateAndAppendColumn(columnName, type, size, required, defValue);
+            throw new InvalidArgumentException("devValue", defValue);
+        // create
+        return this.createAndAppendColumn(columnName, type, size, required, defValue);
     }
 
     /**
@@ -335,7 +311,7 @@ public class DBTable extends DBRowSet implements Cloneable
      */
     public final DBTableColumn addColumn(String columnName, DataType type, double size, boolean required)
     { 
-        return this.crateAndAppendColumn(columnName, type, size, required, null);
+        return this.createAndAppendColumn(columnName, type, size, required, null);
     }
 
     /**
@@ -351,8 +327,9 @@ public class DBTable extends DBRowSet implements Cloneable
      */
     public final DBTableColumn addColumn(String columnName, DataType type, double size, boolean required, Options options)
     {
-        DBTableColumn col = this.crateAndAppendColumn(columnName, type, size, required, null);
-        col.setOptions(options);
+        DBTableColumn col = this.createAndAppendColumn(columnName, type, size, required, null);
+        if (options!=null)
+            col.setOptions(options);
         return col;
     }
 
@@ -369,12 +346,15 @@ public class DBTable extends DBRowSet implements Cloneable
      * @return the new column object 
      */
     public final DBTableColumn addColumn(String columnName, DataType type, double size, boolean required, Options options, Object defValue)
-    { 
+    {
+        // Must not be a class
+        if (options==null)
+            throw new InvalidArgumentException("options", options);
         // defValue must be part of options
         if (defValue!=null && !options.contains(defValue))
             throw new InvalidArgumentException("devValue", defValue);
         // add
-        DBTableColumn col = this.crateAndAppendColumn(columnName, type, size, required, defValue);
+        DBTableColumn col = this.createAndAppendColumn(columnName, type, size, required, defValue);
         col.setOptions(options);
         return col;
     }
@@ -396,8 +376,9 @@ public class DBTable extends DBRowSet implements Cloneable
         {   // Class must be an enum type
             throw new InvalidArgumentException("enumType", enumType);
         }
-        DBTableColumn col = this.crateAndAppendColumn(columnName, type, size, required, null);
-        col.setEnumOptions(enumType);
+        DBTableColumn col = this.createAndAppendColumn(columnName, type, size, required, null);
+        if (enumType!=null)
+            col.setEnumOptions(enumType);
         return col;
     }
 
@@ -409,14 +390,15 @@ public class DBTable extends DBRowSet implements Cloneable
      * @param type the type of the column e.g. integer, text, date
      * @param size the column width
      * @param required true if not null column
-     * @param enumType  defValue the default value
+     * @param enumValue the default value
      * @return the new column object 
      */
     public final DBTableColumn addColumn(String columnName, DataType type, double size, boolean required, Enum<?> enumValue)
     { 
-        Object defValue = ObjectUtils.getEnumValue(enumValue, type.isNumeric());
-        DBTableColumn col = this.crateAndAppendColumn(columnName, type, size, required, defValue);
-        col.setEnumOptions(enumValue.getClass());
+        Object defValue = ObjectUtils.convertValue(type, enumValue);
+        DBTableColumn col = this.createAndAppendColumn(columnName, type, size, required, defValue);
+        if (enumValue!=null)
+            col.setEnumOptions(enumValue.getClass());
         return col;
     }
     
@@ -446,10 +428,13 @@ public class DBTable extends DBRowSet implements Cloneable
     /**
      * Adds a new ForgeinKey table column the column list
      * The foreign table must have a single column foreign key
+     * 
      * @param name the name of the new column
      * @param target the table on which to reference
      * @param required true if the value is required
      * @param options (optional) a set of allowed values for this column
+     * @param cascadeAction the cascade action
+     * 
      * @return the new column
      */
     public DBTableColumn addForeignKey(String name, DBTable target, boolean required, Options options, DBCascadeAction cascadeAction)
@@ -503,13 +488,19 @@ public class DBTable extends DBRowSet implements Cloneable
     }
     
     /**
-     * Adds a Timestamp column to the current table
+     * Adds a Timestamp column to the current table which will be used for optimistic locking.
      * There can only be one timestamp column per table
      * @param name the name of the new column
      * @return the new column
      */
     public DBTableColumn addTimestamp(String name)
     {
+        // check
+        if (this.timestampColumn!=null) {
+            String msg = MessageFormat.format("A Timestamp column ({0}) already exists for table {1}", this.timestampColumn.getName(), this.getName());
+            throw new InvalidOperationException(msg); 
+        }
+        // Add now
         DBTableColumn tsColumn = addColumn(name, DataType.TIMESTAMP, 0, true);
         this.setTimestampColumn(tsColumn);
         return tsColumn;
@@ -575,6 +566,7 @@ public class DBTable extends DBRowSet implements Cloneable
      * Adds an index.
      * 
      * @param index the index to add
+     * @return the Index object
      */
     public DBIndex addIndex(DBIndex index)
     {
@@ -599,7 +591,7 @@ public class DBTable extends DBRowSet implements Cloneable
      * Adds an index.
      * 
      * @param name the index name
-     * @param unique is this a unique index
+     * @param type the index type
      * @param columns the columns indexed by this index
      * 
      * @return the Index object
@@ -619,6 +611,11 @@ public class DBTable extends DBRowSet implements Cloneable
     /**
      * Adds an index.
      * Overload for convenience
+     *
+     * @param name the index name
+     * @param unique flag whether the index is unique
+     * @param columns the index column
+     * @return the index object
      */
     public final DBIndex addIndex(String name, boolean unique, DBColumn... columns)
     {
@@ -639,40 +636,25 @@ public class DBTable extends DBRowSet implements Cloneable
         index.setTable(null);
     }
 
-    /**
-     * Adds a timestamp column to the table used for optimistic locking.
-     * 
-     * @param columnName the column name
-     * 
-     * @return the timestamp table column object
-     */
-    public DBTableColumn addTimestampColumn(String columnName)
-    {
-        DBTableColumn col = addColumn(columnName, DataType.TIMESTAMP, 0, true, DBDatabase.SYSDATE);
-        if (this.timestampColumn!=col)
-            setTimestampColumn(col);    // make sure, this is the timestamp column, even if another one exists
-        return col;
-    }
-
     /**
      * Adds the table's name to the supplied sql command buffer.
      * 
-     * @param buf the SQL-Command
+     * @param sql the SQL-Command
      * @param context the current SQL-Command context
      */
     @Override
-    public void addSQL(StringBuilder buf, long context)
+    public void addSQL(DBSQLBuilder sql, long context)
     {
         // Append Name
         if ((context & CTX_NAME|CTX_FULLNAME)!=0)
         {   // append Qualified Name 
-            db.appendQualifiedName(buf, name, quoteName);
+            db.appendQualifiedName(sql, name, quoteName);
         }
         // Append Alias
         if ((context & CTX_ALIAS)!=0 && alias!=null)
         {    // append alias
-             buf.append(getRenameTablePhrase());
-             buf.append(getAlias());
+             sql.append(getRenameTablePhrase());
+             sql.append(getAlias());
         }
     }
 
@@ -681,7 +663,8 @@ public class DBTable extends DBRowSet implements Cloneable
      * Set this to the specified DBRecord object.
      * 
      * @param record the DBRecord object. contains all fields and the field properties
-     * @param conn a valid connection to the database.
+     * @param initalKey the initial key values
+     * @param deferredInit flag whether to defer initialization
      */
     @Override
     public void createRecord(DBRecordBase record, Object[] initalKey, boolean deferredInit)
@@ -692,9 +675,9 @@ public class DBTable extends DBRowSet implements Cloneable
     
     /**
      * Checks weather a unique constraint is violated when inserting or updating a record.<BR>
-     * <P>
-     * @param id the record's primary key
-     * @param conn a valid JDBC connection
+     * 
+     * @param record the record to check
+     * @return the violated index
      */
     public DBIndex checkUniqueConstraints(DBRecordBase record)
     {
@@ -758,7 +741,7 @@ public class DBTable extends DBRowSet implements Cloneable
      * executeSQL method.
      * 
      * @param key an array of the primary key columns
-     * @param conn a valid connection to the database.
+     * @param context the database context
      */
     @Override
     public void deleteRecord(Object[] key, DBContext context)
@@ -823,7 +806,7 @@ public class DBTable extends DBRowSet implements Cloneable
     /**
      * initializes the Record Default Values
      * @param record the record
-     * @param conn (optional) to allow the dbms handle autogenerated fields
+     * @param fieldInitMode the field initialization mode
      */
     @Override
     protected void initRecordDefaultValues(DBRecordBase record, FieldInitMode fieldInitMode)
