@@ -94,7 +94,7 @@ public class DBUtils implements DBContextAware
 
     /**
      * Returns the current Context
-     * @return
+     * @return the database context
      */
     @Override
     public DBContext getContext()
@@ -105,6 +105,7 @@ public class DBUtils implements DBContextAware
     /**
      * Get single parameter as string (for logging only)
      * @param param the parameter
+     * @return the formatted parameter value
      */
     protected String paramValueToString(Object param)
     {
@@ -142,6 +143,7 @@ public class DBUtils implements DBContextAware
     /**
      * Get all parameters as string (for logging only)
      * @param params the parameter
+     * @return the formatted parameters
      */
     protected String paramsToString(Object[] params)
     {
@@ -488,20 +490,12 @@ public class DBUtils implements DBContextAware
         // check if aggregate 
         if (aggregate)
         {   // For Aggregations: Wrap  
-            DBCommand subCmd = cmd.clone();
-            subCmd.clearOrderBy();
-            String sql = "SELECT COUNT(*) FROM ("+subCmd.getSelect() + ") q";
-            return querySingleInt(sql, subCmd.getParamValues(), 0);
+            String sql = "SELECT COUNT(*) FROM ("+cmd.getSelect(DBCommandExpr.SF_SKIP_ORDER) + ") q";
+            return querySingleInt(sql, cmd.getParamValues(), 0);
         }
-        // find any rowset
-        DBRowSet rs = exprList[0].getRowSet();
-        // create the count command
-        DBCommand countCmd = cmd.clone();
-        countCmd.clearSelect();
-        countCmd.clearOrderBy();
-        countCmd.select(rs.count());
-        // perform query
-        return querySingleInt(countCmd);
+        // simple expression
+        String sql = "SELECT COUNT(*) "+cmd.getSelect(DBCommandExpr.SF_SKIP_SELECT | DBCommandExpr.SF_SKIP_ORDER);
+        return querySingleInt(sql, cmd.getParamValues(), 0);
     }
 
     /**
@@ -565,6 +559,7 @@ public class DBUtils implements DBContextAware
      * @param c the class type for the list 
      * @param <T> the type for the list
      * @param cmd the Command object that contains the select statement
+     * @param result the collection to which to add the result
      * 
      * @return the number of elements that have been added to the collection 
      */
@@ -756,17 +751,37 @@ public class DBUtils implements DBContextAware
      * 
      * @param sqlCmd the SQL-Command
      * @param sqlParams list of query parameter values
+     * @param failOnNoResult if true a QueryNoResultException is thrown when no result otherwise null is returned 
      * 
-     * @return the values of the first row
+     * @return the values of the first row or null
      */
-    public Object[] querySingleRow(String sqlCmd, Object[] sqlParams)
+    public Object[] querySingleRow(String sqlCmd, Object[] sqlParams, boolean failOnNoResult)
     {
         List<Object[]> result = new ArrayList<Object[]>(1);
         queryObjectList(sqlCmd, sqlParams, result, 1);
-        if (result.size()<1)
-            throw new QueryNoResultException(sqlCmd);
+        if (result.size()<1) {
+            // no result
+            if (failOnNoResult)
+                throw new QueryNoResultException(sqlCmd);
+            // null
+            return null;
+        }
         return result.get(0);
     }
+
+    /**
+     * Returns all values of the first row of a sql-query as an array.
+     * If the query does not return a result a QueryNoResultException is thrown
+     * 
+     * @param sqlCmd the SQL-Command
+     * @param sqlParams list of query parameter values
+     * 
+     * @return the values of the first row
+     */
+    public final Object[] querySingleRow(String sqlCmd, Object[] sqlParams)
+    {
+        return querySingleRow(sqlCmd, sqlParams, true);
+    }
     
     /**
      * Returns all values of the first row of a sql-query as an array.
@@ -791,13 +806,24 @@ public class DBUtils implements DBContextAware
         log.warn("Query Result was limited to {} by MAX_QUERY_ROWS", MAX_QUERY_ROWS);
         log.warn("********************************************************");
     }
+
+    /**
+     * Called when a query has no result
+     * @param cmd the command
+     * @param resultType the result type class
+     */
+    protected void handleQueryNoResult(DBCommandExpr cmd, Class<?> resultType)
+    {
+        // throw new QueryNoResultException(cmd.getSelect());
+        log.debug("The query for \"{}\" returned no result.", resultType.getName());
+    }
     
     /**
      * Crates a default DataListFactory for a DataListEntry class
      * The DataListEntry class must provide the following constructor
      *      DataListEntry(DataListFactory&lt;? extends DataListEntry&gt; head, int rownum, Object values[])
      * @param entryClass the entryClass for which to create the list head 
-     * @return
+     * @return the data list factory
      */
     protected <T extends DataListEntry> DataListFactory<T> createDefaultDataListFactory(Class<T> entryClass, DataListHead head) 
     {
@@ -807,6 +833,7 @@ public class DBUtils implements DBContextAware
     /**
      * Crates a default DataListHead for a DataListEntry class
      * @param cmd the cmd for which to create the DataListHead
+     * @param entryClass the entry type class
      * @return the DataListHead instance
      */
     protected DataListHead createDefaultDataListHead(DBCommandExpr cmd, Class<? extends DataListEntry> entryClass) 
@@ -886,6 +913,10 @@ public class DBUtils implements DBContextAware
 
     /**
      * Queries a list of DataListEntry items
+     * @param cmd the query command
+     * @param entryClass the entry type class
+     * @param head the list head
+     * @return the data list
      */
     public final <T extends DataListEntry> List<T> queryDataList(DBCommandExpr cmd, Class<T> entryClass, DataListHead head)
     {
@@ -894,6 +925,9 @@ public class DBUtils implements DBContextAware
     
     /**
      * Queries a list of DataListEntry items
+     * @param cmd the query command
+     * @param entryClass the entry type class
+     * @return the data list
      */
     public final <T extends DataListEntry> List<T> queryDataList(DBCommandExpr cmd, Class<T> entryClass)
     {
@@ -902,6 +936,11 @@ public class DBUtils implements DBContextAware
     
     /**
      * Queries a list of DataListEntry items
+     * @param cmd the query command
+     * @param entryClass the entry type class
+     * @param first the first record to add
+     * @param maxItems the maximum number of records to add
+     * @return the data list
      */
     public final <T extends DataListEntry> List<T> queryDataList(DBCommandExpr cmd, Class<T> entryClass, int first, int maxItems)
     {
@@ -910,6 +949,8 @@ public class DBUtils implements DBContextAware
     
     /**
      * Queries a list of DataListEntry items
+     * @param cmd the query command
+     * @return the data list
      */
     public final List<DataListEntry> queryDataList(DBCommandExpr cmd)
     {
@@ -918,23 +959,29 @@ public class DBUtils implements DBContextAware
     
     /**
      * Queries a list of DataListEntry items
+     * @param cmd the query command
+     * @param first the first record to add
+     * @param maxItems the maximum number of records to add
+     * @return the data list
      */
     public final List<DataListEntry> queryDataList(DBCommandExpr cmd, int first, int maxItems)
     {
         return queryDataList(cmd, DataListEntry.class, first, maxItems);
     }
-
+    
     /**
      * Queries a single DataListEntry item
-     * @param failOnNoResult flag whether to fail on empty resultset
+     * @param cmd the query command
+     * @param entryClass the result class
+     * @param head the list head
+     * @return the data entry
      */
-    public final <T extends DataListEntry> T queryDataEntry(DBCommandExpr cmd, Class<T> entryClass, boolean failOnNoResult)
+    public final <T extends DataListEntry> T queryDataEntry(DBCommandExpr cmd, Class<T> entryClass, DataListHead head)
     {
-        DataListHead head = createDefaultDataListHead(cmd, entryClass);
         List<T> dle = queryDataList(cmd, createDefaultDataListFactory(entryClass, head), 0, 1);
         if (dle.isEmpty())
-        {   if (failOnNoResult)
-                throw new QueryNoResultException(cmd.getSelect());
+        {   // No result
+            handleQueryNoResult(cmd, entryClass);
             return null;
         }
         return dle.get(0);
@@ -942,14 +989,37 @@ public class DBUtils implements DBContextAware
     
     /**
      * Queries a single DataListEntry item
+     * @param cmd the query command
+     * @param entryClass the result class
+     * @return the data entry
      */
     public final <T extends DataListEntry> T queryDataEntry(DBCommandExpr cmd, Class<T> entryClass)
     {
-        return queryDataEntry(cmd, entryClass, true);
+        DataListHead head = createDefaultDataListHead(cmd, entryClass);
+        return queryDataEntry(cmd, entryClass, head);
+    }
+
+    /**
+     * Queries a single DataListEntry item
+     * Deprecated. Please consider calling without the "failOnNoResult" parameter and overriding handleQueryNoResult()
+     * @param cmd the query command
+     * @param entryClass the result class
+     * @param failOnNoResult flag whether to fail on empty resultset
+     * @return the data entry
+     */
+    @Deprecated
+    public final <T extends DataListEntry> T queryDataEntry(DBCommandExpr cmd, Class<T> entryClass, boolean failOnNoResult)
+    {
+        T result = queryDataEntry(cmd, entryClass);
+        if (result==null && failOnNoResult)
+            throw new QueryNoResultException(cmd.getSelect());
+        return result;
     }
     
     /**
      * Queries a single DataListEntry item
+     * @param cmd the query command
+     * @return the data entry
      */
     public final DataListEntry queryDataEntry(DBCommandExpr cmd)
     {
@@ -961,7 +1031,7 @@ public class DBUtils implements DBContextAware
      * The DBRecord class must provide the following constructor
      *      DBRecord(DBContext context, DBRowSet rowset)
      * @param recordClass the recordClass for which to create the list head 
-     * @return
+     * @return the record factory
      */
     protected <R extends DBRecordBase> DBRecordListFactory<R> createDefaultRecordListFactory(Class<R> recordClass, DBRowSet rowset) 
     {
@@ -1016,14 +1086,9 @@ public class DBUtils implements DBContextAware
                 R entry = factory.newRecord(rownum, r);
                 if (entry==null)
                     continue;
-                // check
-                if (entry.isValid())
-                {   // add entry
-                    list.add(entry);
-                    rownum++;
-                }
-                else
-                    log.trace("Record {} is not valid thus it will not be added to the RecordListQuery.", rownum);
+                // add entry
+                list.add(entry);
+                rownum++;
                 // Decrease count
                 if (maxCount > 0)
                     maxCount--;
@@ -1047,6 +1112,7 @@ public class DBUtils implements DBContextAware
      * Executes a query and returns a list of DBRecord items
      * @param cmd the command holding the constraints and order or the query
      * @param rowset the rowset for which to query the records
+     * @param recordType the record class
      * @return the list of DBRecord items
      */
     public final <R extends DBRecordBase> List<R> queryRecordList(DBCommand cmd, DBRowSet rowset, Class<R> recordType)
@@ -1128,7 +1194,7 @@ public class DBUtils implements DBContextAware
      * @param parent the parent object for the created beans (optional)
      * @param first the first row
      * @param pageSize the maximum number of items to add to the list or -1 (default) for all
-     * @return
+     * @return the bean list
      */
     public <T> List<T> queryBeanList(DBCommandExpr cmd, DBBeanListFactory<T> factory, Object parent, int first, int pageSize)
     {
@@ -1246,7 +1312,13 @@ public class DBUtils implements DBContextAware
         {   // prepare
             factory.prepareQuery(cmd, context);
             // Runquery
-            r.getRecordData(cmd);
+            r.open(cmd);
+            // Get First Record
+            if (!r.moveNext())
+            {   // No Result
+                handleQueryNoResult(cmd, factory.getBeanType());
+                return null;
+            }
             // add data
             T item = factory.newItem(-1, r);
             // post processing
