@@ -27,6 +27,7 @@ import org.apache.empire.data.DataType;
 import org.apache.empire.db.DBColumn;
 import org.apache.empire.db.DBColumnExpr;
 import org.apache.empire.db.DBDatabase;
+import org.apache.empire.db.DBRowSet;
 import org.apache.empire.db.DBSQLBuilder;
 import org.apache.empire.xml.XMLUtil;
 import org.slf4j.Logger;
@@ -39,7 +40,6 @@ import org.w3c.dom.Element;
  * <P>
  * There is no need to explicitly create instances of this class.<BR>
  * Instead use {@link DBDatabase#getValueExpr(String)} or one of it's overloads
- * <P>
  *
  */
 public class DBValueExpr extends DBColumnExpr
@@ -180,16 +180,18 @@ public class DBValueExpr extends DBColumnExpr
     }
 
     /**
-     * Returns the expression the source column.
+     * Returns null.
+     * @return null
      */
     @Override
-    public DBColumn getSourceColumn()
+    public DBRowSet getRowSet()
     {
         return null;
     }
 
     /**
-     * Returns the expression the update column.
+     * Returns null.
+     * @return null
      */
     @Override
     public DBColumn getUpdateColumn()
