@@ -33,6 +33,7 @@ import org.apache.empire.db.DBDDLGenerator;
 import org.apache.empire.db.DBDDLGenerator.DDLActionType;
 import org.apache.empire.db.DBDatabase;
 import org.apache.empire.db.DBObject;
+import org.apache.empire.db.DBSQLBuilder;
 import org.apache.empire.db.DBSQLScript;
 import org.apache.empire.db.DBTable;
 import org.apache.empire.db.DBTableColumn;
@@ -130,7 +131,7 @@ public class DBMSHandlerPostgreSQL extends DBMSHandlerBase
     @Override
     public DBCommandPostgres createCommand(boolean autoPrepareStmt)
     {   // create command object
-        return new DBCommandPostgres(autoPrepareStmt);
+        return new DBCommandPostgres(this, autoPrepareStmt);
     }
 
     /**
@@ -156,7 +157,7 @@ public class DBMSHandlerPostgreSQL extends DBMSHandlerBase
     
     /**
      * Gets an sql phrase template for this database system.<br>
-     * @see DBMSHandler#getSQLPhrase(int)
+     * @see DBMSHandler#getSQLPhrase(DBSqlPhrase)
      * @return the phrase template
      */
     @Override
@@ -262,7 +263,7 @@ public class DBMSHandlerPostgreSQL extends DBMSHandlerBase
 
     
     /**
-     * @see DBMSHandler#getNextSequenceValue(DBDatabase, String, int, Connection)
+     * @see DBMSHandlerBase#getNextSequenceValue(DBDatabase, String, int, Connection)
      */
     @Override
     public Object getNextSequenceValue(DBDatabase db, String seqName, int minValue, Connection conn)
@@ -279,7 +280,7 @@ public class DBMSHandlerPostgreSQL extends DBMSHandlerBase
     }
 
     /**
-     * @see DBMSHandler#getNextSequenceValueExpr(DBTableColumn col)
+     * @see DBMSHandlerBase#getNextSequenceValueExpr(DBTableColumn col)
      */
     @Override
     public DBColumnExpr getNextSequenceValueExpr(DBTableColumn column)
@@ -289,7 +290,7 @@ public class DBMSHandlerPostgreSQL extends DBMSHandlerBase
         {
             throw new InvalidArgumentException("column", column);
         }
-        StringBuilder sql = new StringBuilder(80);
+        DBSQLBuilder sql = createSQLBuilder();
         sql.append("nextval('");
         column.getDatabase().appendQualifiedName(sql, seqName, false);
         sql.append("')");
